@@ -1,30 +1,35 @@
-/*****************************************************************************/
-/*  misc.c - general purpose routines                                        */
-/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
-/*                                                                           */
-/*  This program is free software; you can redistribute it and/or modify     */
-/*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 2 of the License, or        */
-/*  (at your option) any later version.                                      */
-/*                                                                           */
-/*  This program is distributed in the hope that it will be useful,          */
-/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
-/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
-/*  GNU General Public License for more details.                             */
-/*                                                                           */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA      */
-/*****************************************************************************/
+/***********************************************************************************/
+/*  misc.c - general purpose routines                                              */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                        */
+/*                                                                                 */
+/*  Permission is hereby granted, free of charge, to any person obtaining a copy   */
+/*  of this software and associated documentation files (the "Software"), to deal  */
+/*  in the Software without restriction, including without limitation the rights   */
+/*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      */
+/*  copies of the Software, and to permit persons to whom the Software is          */
+/*  furnished to do so, subject to the following conditions:                       */
+/*                                                                                 */
+/*  The above copyright notice and this permission notice shall be included in all */
+/*  copies or substantial portions of the Software.                                */
+/*                                                                                 */
+/*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     */
+/*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       */
+/*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    */
+/*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         */
+/*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  */
+/*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  */
+/*  SOFTWARE.                                                                      */
+/***********************************************************************************/
 
 #include "gftp.h"
 #include "options.h"
+#include <fnmatch.h>
 
 #ifdef HAVE_INTL_PRINTF
 
-char *
-insert_commas (off_t number, char *dest_str, size_t dest_len)
+char * insert_commas (off_t number, char *dest_str, size_t dest_len)
 {
+  //DEBUG_PRINT_FUNC
   if (dest_str != NULL)
     g_snprintf (dest_str, dest_len, GFTP_OFF_T_INTL_PRINTF_MOD, number);
   else
@@ -35,9 +40,9 @@ insert_commas (off_t number, char *dest_str, size_t dest_len)
 
 #else
 
-char *
-insert_commas (off_t number, char *dest_str, size_t dest_len)
+char * insert_commas (off_t number, char *dest_str, size_t dest_len)
 {
+  //DEBUG_PRINT_FUNC
   char *frompos, *topos, src[50], *dest;
   size_t num, rem, srclen;
   size_t len, i;
@@ -105,9 +110,9 @@ insert_commas (off_t number, char *dest_str, size_t dest_len)
 
 #endif
 
-char *
-alltrim (char *str)
+char * alltrim (char *str)
 {
+  //DEBUG_PRINT_FUNC
   char *pos, *newpos;
   int diff;
 
@@ -133,9 +138,9 @@ alltrim (char *str)
 }
 
 
-char *
-gftp_expand_path (gftp_request * request, const char *src)
+char * gftp_expand_path (gftp_request * request, const char *src)
 {
+  //DEBUG_PRINT_FUNC
   char *str, *pos, *endpos, *prevpos, *newstr, *tempstr, *ntoken,
        tempchar;
   struct passwd *pw;
@@ -237,70 +242,30 @@ gftp_expand_path (gftp_request * request, const char *src)
 }
 
 
-void
-make_nonnull (char **str)
+int gftp_match_filespec (gftp_request * request, const char *filename,
+                         const char *filespec)
 {
-  if (*str == NULL)
-    *str = g_malloc0 (1);
-}
-
-
-/* FIXME - Possible use the libpcre library. If it isn't used, then clean
-   this function up some more. */
-int
-gftp_match_filespec (gftp_request * request, const char *filename,
-                     const char *filespec)
-{
-  const char *filepos, *wcpos, *pos;
-  char search_str[20], *newpos;
-  intptr_t show_hidden_files;
-  size_t len, curlen;
+   //DEBUG_PRINT_FUNC
+   intptr_t show_hidden_files;
   
-  if (filename == NULL || *filename == '\0' || 
-      filespec == NULL || *filespec == '\0') 
-    return (1);
-
-  gftp_lookup_request_option (request, "show_hidden_files", &show_hidden_files);
-  if (!show_hidden_files && *filename == '.' && strcmp (filename, "..") != 0)
-    return (0);
-
-  filepos = filename;
-  wcpos = filespec;
-  while (1)
-    {
-      if (*wcpos == '\0') 
-        return (1);
-      else if (*filepos == '\0') 
-        return (0);
-      else if (*wcpos == '?') 
-        {
-          wcpos++;
-          filepos++;
-        }
-      else if (*wcpos == '*' && *(wcpos+1) == '\0') 
-        return (1);
-      else if (*wcpos == '*') 
-        {
-          len = sizeof (search_str);
-          for (pos = wcpos + 1, newpos = search_str, curlen = 0;
-               *pos != '*' && *pos != '?' && *pos != '\0' && curlen < len;
-               curlen++, *newpos++ = *pos++);
-          *newpos = '\0';
-
-          if ((filepos = strstr (filepos, search_str)) == NULL)
-            return (0);
-          wcpos += curlen + 1;
-          filepos += curlen;
-        }
-      else if(*wcpos++ != *filepos++) 
-        return (0);
-    }
+   if (!filename || !*filename || !filespec || !*filespec) {
+      return (1);
+   }
+   gftp_lookup_request_option (request, "show_hidden_files", &show_hidden_files);
+   if (!show_hidden_files && *filename == '.' && strcmp (filename, "..") != 0) {
+      return (0);
+   }
+   if (fnmatch (filespec, filename, 0) == 0) {
+      return 1;
+   } else {
+      return 0;
+   }
 }
 
 
-static void
-gftp_info (void)
+static void gftp_info (void)
 {
+  DEBUG_PRINT_FUNC
   int i;
 
   printf ("%s\n", gftp_version);
@@ -321,7 +286,7 @@ gftp_info (void)
   printf ("#define _FILE_OFFSET_BITS %d\n", _FILE_OFFSET_BITS);
 #endif
 
-  printf ("sizeof (off_t) = %i\n", sizeof (off_t));
+  printf ("sizeof (off_t) = %i\n", (int)sizeof (off_t));
 
 #ifdef HAVE_INTL_PRINTF
   printf ("#define HAVE_INTL_PRINTF\n");
@@ -366,9 +331,9 @@ gftp_info (void)
 }
 
 
-int
-gftp_parse_command_line (int *argc, char ***argv)
+int gftp_parse_command_line (int *argc, char ***argv)
 {
+  DEBUG_PRINT_FUNC
   if (*argc > 1)
     {
       if (strcmp (argv[0][1], "--help") == 0 || 
@@ -393,23 +358,21 @@ gftp_parse_command_line (int *argc, char ***argv)
 }
 
 
-void
-gftp_usage (void)
+void gftp_usage (void)
 {
+  DEBUG_PRINT_FUNC
   printf (_("usage: gftp " GFTP_URL_USAGE "\n"));
   exit (0);
 }
 
 
-gint
-string_hash_compare (gconstpointer path1, gconstpointer path2)
+gint string_hash_compare (gconstpointer path1, gconstpointer path2)
 {
   return (strcmp ((char *) path1, (char *) path2) == 0);
 }
 
 
-guint
-string_hash_function (gconstpointer key)
+guint string_hash_function (gconstpointer key)
 {
   guint ret;
   int i;
@@ -422,23 +385,21 @@ string_hash_function (gconstpointer key)
 }
 
 
-gint
-uint_hash_compare (gconstpointer path1, gconstpointer path2)
+gint uint_hash_compare (gconstpointer path1, gconstpointer path2)
 {
   return (GPOINTER_TO_UINT (path1) == GPOINTER_TO_UINT (path2));
 }
 
 
-guint
-uint_hash_function (gconstpointer key)
+guint uint_hash_function (gconstpointer key)
 {
   return (GPOINTER_TO_UINT (key));
 }
 
 
-void
-free_file_list (GList * filelist)
+void free_file_list (GList * filelist)
 {
+  DEBUG_PRINT_FUNC
   gftp_file * tempfle;
   GList * templist;
 
@@ -455,9 +416,9 @@ free_file_list (GList * filelist)
 }
 
 
-gftp_file *
-copy_fdata (gftp_file * fle)
+gftp_file * copy_fdata (gftp_file * fle)
 {
+  DEBUG_PRINT_FUNC
   gftp_file * newfle;
 
   newfle = g_malloc0 (sizeof (*newfle));
@@ -480,10 +441,10 @@ copy_fdata (gftp_file * fle)
 }
 
 
-int
-compare_request (gftp_request * request1, gftp_request * request2,
-                 int compare_dirs)
+int compare_request (gftp_request * request1, gftp_request * request2,
+                     int compare_dirs)
 {
+  DEBUG_PRINT_FUNC
   char *strarr[3][2];
   int i, ret;
 
@@ -526,9 +487,9 @@ compare_request (gftp_request * request1, gftp_request * request2,
 }
 
 
-gftp_transfer *
-gftp_tdata_new (void)
+gftp_transfer * gftp_tdata_new (void)
 {
+  DEBUG_PRINT_FUNC
   gftp_transfer * tdata;
 
   tdata = g_malloc0 (sizeof (*tdata));
@@ -540,43 +501,37 @@ gftp_tdata_new (void)
 }
 
 
-void
-free_tdata (gftp_transfer * tdata)
+void free_tdata (gftp_transfer * tdata)
 {
-  if (tdata->fromreq != NULL)
-    gftp_request_destroy (tdata->fromreq, 1);
-  if (tdata->toreq != NULL)
-    gftp_request_destroy (tdata->toreq, 1);
+  DEBUG_PRINT_FUNC
+  if (tdata->fromreq)   gftp_request_destroy (tdata->fromreq, 1);
+  if (tdata->toreq)     gftp_request_destroy (tdata->toreq, 1);
+  if (tdata->thread_id) g_free (tdata->thread_id);
   free_file_list (tdata->files);
-  if (tdata->thread_id != NULL)
-    g_free (tdata->thread_id);
   g_free (tdata);
 }
 
 
-gftp_request * 
-gftp_copy_request (gftp_request * req)
+gftp_request * gftp_copy_request (gftp_request * req)
 {
+  DEBUG_PRINT_FUNC
   gftp_request * newreq;
 
   newreq = gftp_request_new ();
 
-  if (req->hostname)
-    newreq->hostname = g_strdup (req->hostname);
-  if (req->username)
-    newreq->username = g_strdup (req->username);
-  if (req->password)
-    newreq->password = g_strdup (req->password);
-  if (req->account)
-    newreq->account = g_strdup (req->account);
-  if (req->directory)
-    newreq->directory = g_strdup (req->directory);
-  if (req->url_prefix)
-    newreq->url_prefix = g_strdup (req->url_prefix);
+  if (req->hostname)   newreq->hostname   = g_strdup (req->hostname);
+  if (req->username)   newreq->username   = g_strdup (req->username);
+  if (req->password)   newreq->password   = g_strdup (req->password);
+  if (req->account)    newreq->account    = g_strdup (req->account);
+  if (req->directory)  newreq->directory  = g_strdup (req->directory);
+  if (req->url_prefix) newreq->url_prefix = req->url_prefix; /* this is not freed */
+
   newreq->port = req->port;
   newreq->use_proxy = req->use_proxy;
   newreq->logging_function = req->logging_function;
   newreq->ai_family = req->ai_family;
+  newreq->ai_socktype = req->ai_socktype;
+  newreq->use_udp = req->use_udp;
 
   if (req->remote_addr == NULL)
     {
@@ -608,8 +563,7 @@ gftp_copy_request (gftp_request * req)
 }
 
 
-static gint
-gftp_file_sort_function_as (gconstpointer a, gconstpointer b)
+static gint gftp_file_sort_function_as (gconstpointer a, gconstpointer b)
 {
   const gftp_file * f1, * f2;
 
@@ -619,8 +573,7 @@ gftp_file_sort_function_as (gconstpointer a, gconstpointer b)
 }
 
 
-static gint
-gftp_file_sort_function_ds (gconstpointer a, gconstpointer b)
+static gint gftp_file_sort_function_ds (gconstpointer a, gconstpointer b)
 {
   const gftp_file * f1, * f2;
   gint ret;
@@ -632,8 +585,7 @@ gftp_file_sort_function_ds (gconstpointer a, gconstpointer b)
 }
 
 
-static gint
-gftp_user_sort_function_as (gconstpointer a, gconstpointer b)
+static gint gftp_user_sort_function_as (gconstpointer a, gconstpointer b)
 {
   const gftp_file * f1, * f2;
 
@@ -643,8 +595,7 @@ gftp_user_sort_function_as (gconstpointer a, gconstpointer b)
 }
 
 
-static gint
-gftp_user_sort_function_ds (gconstpointer a, gconstpointer b)
+static gint gftp_user_sort_function_ds (gconstpointer a, gconstpointer b)
 {
   const gftp_file * f1, * f2;
   gint ret;
@@ -656,8 +607,7 @@ gftp_user_sort_function_ds (gconstpointer a, gconstpointer b)
 }
 
 
-static gint
-gftp_group_sort_function_as (gconstpointer a, gconstpointer b)
+static gint gftp_group_sort_function_as (gconstpointer a, gconstpointer b)
 {
   const gftp_file * f1, * f2;
 
@@ -667,8 +617,7 @@ gftp_group_sort_function_as (gconstpointer a, gconstpointer b)
 }
 
 
-static gint
-gftp_group_sort_function_ds (gconstpointer a, gconstpointer b)
+static gint gftp_group_sort_function_ds (gconstpointer a, gconstpointer b)
 {
   const gftp_file * f1, * f2;
   gint ret;
@@ -680,8 +629,7 @@ gftp_group_sort_function_ds (gconstpointer a, gconstpointer b)
 }
 
 
-static gint
-gftp_attribs_sort_function_as (gconstpointer a, gconstpointer b)
+static gint gftp_attribs_sort_function_as (gconstpointer a, gconstpointer b)
 {
   const gftp_file * f1, * f2;
 
@@ -696,8 +644,7 @@ gftp_attribs_sort_function_as (gconstpointer a, gconstpointer b)
 }
 
 
-static gint
-gftp_attribs_sort_function_ds (gconstpointer a, gconstpointer b)
+static gint gftp_attribs_sort_function_ds (gconstpointer a, gconstpointer b)
 {
   const gftp_file * f1, * f2;
 
@@ -712,8 +659,7 @@ gftp_attribs_sort_function_ds (gconstpointer a, gconstpointer b)
 }
 
 
-static gint
-gftp_size_sort_function_as (gconstpointer a, gconstpointer b)
+static gint gftp_size_sort_function_as (gconstpointer a, gconstpointer b)
 {
   const gftp_file * f1, * f2;
 
@@ -728,8 +674,7 @@ gftp_size_sort_function_as (gconstpointer a, gconstpointer b)
 }
 
 
-static gint
-gftp_size_sort_function_ds (gconstpointer a, gconstpointer b)
+static gint gftp_size_sort_function_ds (gconstpointer a, gconstpointer b)
 {
   const gftp_file * f1, * f2;
 
@@ -744,8 +689,7 @@ gftp_size_sort_function_ds (gconstpointer a, gconstpointer b)
 }
 
 
-static gint
-gftp_datetime_sort_function_as (gconstpointer a, gconstpointer b)
+static gint gftp_datetime_sort_function_as (gconstpointer a, gconstpointer b)
 {
   const gftp_file * f1, * f2;
 
@@ -760,8 +704,7 @@ gftp_datetime_sort_function_as (gconstpointer a, gconstpointer b)
 }
 
 
-static gint
-gftp_datetime_sort_function_ds (gconstpointer a, gconstpointer b)
+static gint gftp_datetime_sort_function_ds (gconstpointer a, gconstpointer b)
 {
   const gftp_file * f1, * f2;
 
@@ -776,9 +719,9 @@ gftp_datetime_sort_function_ds (gconstpointer a, gconstpointer b)
 }
 
 
-GList *
-gftp_sort_filelist (GList * filelist, int column, int asds)
+GList * gftp_sort_filelist (GList * filelist, int column, int asds)
 {
+  DEBUG_PRINT_FUNC
   GList * files, * dirs, * dotdot, * tempitem, * insitem;
   GCompareFunc sortfunc;
   gftp_file * tempfle;
@@ -905,80 +848,19 @@ gftp_gen_ls_string (gftp_request * request, gftp_file * fle,
 }
 
 
-char *
-base64_encode (char *str)
-{
-
-/* The standard to Base64 encoding can be found in RFC2045 */
-
-  char *newstr, *newpos, *fillpos, *pos;
-  unsigned char table[64], encode[3];
-  size_t slen, num;
-  int i;
-
-  for (i = 0; i < 26; i++)
-    {
-      table[i] = 'A' + i;
-      table[i + 26] = 'a' + i;
-    }
-
-  for (i = 0; i < 10; i++)
-    table[i + 52] = '0' + i;
-
-  table[62] = '+';
-  table[63] = '/';
-
-  slen = strlen (str);
-  num = slen / 3;
-  if (slen % 3 > 0)
-    num++;
-
-  newstr = g_malloc0 ((gulong) num * 4 + 1);
-  newstr[num * 4] = '\0';
-  newpos = newstr;
-
-  pos = str;
-  while (*pos != '\0')
-    {
-      memset (encode, 0, sizeof (encode));
-      for (i = 0; i < 3 && *pos != '\0'; i++)
-          encode[i] = *pos++;
-
-      fillpos = newpos;
-      *newpos++ = table[encode[0] >> 2];
-      *newpos++ = table[(encode[0] & 3) << 4 | encode[1] >> 4];
-      *newpos++ = table[(encode[1] & 0xF) << 2 | encode[2] >> 6];
-      *newpos++ = table[encode[2] & 0x3F];
-      while (i < 3)
-          fillpos[++i] = '=';
-    }
-  return (newstr);
-}
-
-
-void
-gftp_free_bookmark (gftp_bookmarks_var * entry, int free_node)
+void gftp_free_bookmark (gftp_bookmarks_var * entry, int free_node)
 {
+  DEBUG_PRINT_FUNC
   gftp_bookmarks_var * tempentry;
 
-  if (entry->path)
-    g_free (entry->path);
-  if (entry->oldpath)
-    g_free (entry->oldpath);
-  if (entry->hostname)
-    g_free (entry->hostname);
-  if (entry->remote_dir)
-    g_free (entry->remote_dir);
-  if (entry->local_dir)
-    g_free (entry->local_dir);
-  if (entry->user)
-    g_free (entry->user);
-  if (entry->pass)
-    g_free (entry->pass);
-  if (entry->acct)
-    g_free (entry->acct);
-  if (entry->protocol)
-    g_free (entry->protocol);
+  if (entry->path)       g_free (entry->path);
+  if (entry->hostname)   g_free (entry->hostname);
+  if (entry->remote_dir) g_free (entry->remote_dir);
+  if (entry->local_dir)  g_free (entry->local_dir);
+  if (entry->user)       g_free (entry->user);
+  if (entry->pass)       g_free (entry->pass);
+  if (entry->acct)       g_free (entry->acct);
+  if (entry->protocol)   g_free (entry->protocol);
 
   if (entry->local_options_vars != NULL)
     {
@@ -1002,9 +884,9 @@ gftp_free_bookmark (gftp_bookmarks_var * entry, int free_node)
 }
 
 
-void
-gftp_shutdown (void)
+void gftp_shutdown (void)
 {
+  DEBUG_PRINT_FUNC
 #ifdef WITH_DMALLOC
   gftp_config_vars * cv;
   GList * templist;
@@ -1045,9 +927,9 @@ gftp_shutdown (void)
 }
 
 
-char *
-gftp_build_path (gftp_request * request, const char *first_element, ...) 
+char * gftp_build_path (gftp_request * request, const char *first_element, ...) 
 {
+  //DEBUG_PRINT_FUNC
   const char *element;
   size_t len, retlen;
   int add_separator;
@@ -1080,11 +962,9 @@ gftp_build_path (gftp_request * request, const char *first_element, ...)
       retlen += len;
       ret = g_realloc (ret, (gulong) retlen + 1);
 
-      /* Don't append a / for VMS servers... */
-      if (add_separator &&
-          (request == NULL || request->server_type != GFTP_DIRTYPE_VMS))
-        strncat (ret, "/", retlen);
-
+      if (add_separator) {
+         strncat (ret, "/", retlen);
+      }
       strncat (ret, element, retlen);
     }
 
@@ -1092,9 +972,9 @@ gftp_build_path (gftp_request * request, const char *first_element, ...)
 }
 
 
-void
-gftp_locale_init (void)
+void gftp_locale_init (void)
 {
+  DEBUG_PRINT_FUNC
 #ifdef HAVE_GETTEXT
 
   setlocale (LC_ALL, "");
@@ -1113,9 +993,9 @@ gftp_locale_init (void)
 */
 
 
-char * 
-gftp_scramble_password (const char *password)
+char * gftp_scramble_password (const char *password)
 {
+  //DEBUG_PRINT_FUNC
   char *newstr, *newpos;
 
   if (strcmp (password, "@EMAIL@") == 0)
@@ -1138,9 +1018,9 @@ gftp_scramble_password (const char *password)
 }
 
 
-char *
-gftp_descramble_password (const char *password)
+char * gftp_descramble_password (const char *password)
 {
+  //DEBUG_PRINT_FUNC
   const char *passwordpos;
   char *newstr, *newpos;
   int error;
@@ -1179,9 +1059,9 @@ gftp_descramble_password (const char *password)
 }
 
 
-int
-gftp_get_transfer_action (gftp_request * request, gftp_file * fle)
+int gftp_get_transfer_action (gftp_request * request, gftp_file * fle)
 {
+  DEBUG_PRINT_FUNC
   intptr_t overwrite_default;
 
   gftp_lookup_request_option (request, "overwrite_default", &overwrite_default);
@@ -1203,9 +1083,9 @@ gftp_get_transfer_action (gftp_request * request, gftp_file * fle)
 }
 
 
-char *
-gftp_get_share_dir (void)
+char * gftp_get_share_dir (void)
 {
+  DEBUG_PRINT_FUNC
   static char *gftp_share_dir = NULL;
   char *envval;
 
@@ -1222,8 +1102,9 @@ gftp_get_share_dir (void)
   return (gftp_share_dir);
 }
 
-void
-gftp_format_file_size(off_t bytes, char *out_buffer, size_t buffer_size) {
+void gftp_format_file_size(off_t bytes, char *out_buffer, size_t buffer_size)
+{
+  //DEBUG_PRINT_FUNC
   char tmp[20];
   char *pointzero = 0;
   off_t TB = 1024LL * 1024LL * 1024LL * 1024LL;
@@ -1255,3 +1136,51 @@ gftp_format_file_size(off_t bytes, char *out_buffer, size_t buffer_size) {
   }
 }
 
+
+char * str_get_next_line_pointer (char * buf, char **pos, int *line_ending)
+{
+   // modified buf, new lines become '\0'
+   // start: *pos must be NULL, buf = line1\r\nline2\r\n, etc
+   // empty lines are treated as eof
+   char * ret = NULL;
+   char * nextpos = NULL;
+   char * p;
+   if (buf == NULL) {
+      return NULL;
+   }
+   if (*pos == NULL) {
+      *pos = buf;
+   }
+   p = *pos;
+   if (!*p) {
+      return NULL;
+   }
+   nextpos = strchr (*pos, '\n');
+   if (nextpos && nextpos != *pos) {
+      *nextpos = 0;
+      if (*(nextpos-1) == '\r') {
+         *(nextpos-1) = 0;
+      }
+      nextpos++;
+      ret = *pos;
+      if (line_ending) {
+         *line_ending = 1;
+      }
+   } else {
+      // handle last line without \r\n
+      ret = *pos;
+      p = *pos;
+      if (*p) {
+         while (*p) p++;
+         *pos = p;
+      }
+      if (line_ending) {
+         *line_ending = 0;
+      }
+   }
+
+   if (nextpos && nextpos != *pos) {
+      *pos = nextpos;
+   }
+   return ret;
+}
