@@ -1,64 +1,60 @@
-/*****************************************************************************/
-/*  protocols.c - Skeleton functions for the protocols gftp supports         */
-/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
-/*                                                                           */
-/*  This program is free software; you can redistribute it and/or modify     */
-/*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 2 of the License, or        */
-/*  (at your option) any later version.                                      */
-/*                                                                           */
-/*  This program is distributed in the hope that it will be useful,          */
-/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
-/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
-/*  GNU General Public License for more details.                             */
-/*                                                                           */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA      */
-/*****************************************************************************/
+/***********************************************************************************/
+/*  protocols.c - Skeleton functions for the protocols gftp supports               */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                        */
+/*                                                                                 */
+/*  Permission is hereby granted, free of charge, to any person obtaining a copy   */
+/*  of this software and associated documentation files (the "Software"), to deal  */
+/*  in the Software without restriction, including without limitation the rights   */
+/*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      */
+/*  copies of the Software, and to permit persons to whom the Software is          */
+/*  furnished to do so, subject to the following conditions:                       */
+/*                                                                                 */
+/*  The above copyright notice and this permission notice shall be included in all */
+/*  copies or substantial portions of the Software.                                */
+/*                                                                                 */
+/*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     */
+/*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       */
+/*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    */
+/*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         */
+/*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  */
+/*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  */
+/*  SOFTWARE.                                                                      */
+/***********************************************************************************/
 
 #include "gftp.h"
 
 gftp_request *
 gftp_request_new (void)
 {
+  DEBUG_PRINT_FUNC
   gftp_request *request;
 
   request = g_malloc0 (sizeof (*request));
   request->datafd = -1;
   request->cachefd = -1;
-  request->server_type = GFTP_DIRTYPE_OTHER;
   return (request);
 }
 
 
-void
-gftp_request_destroy (gftp_request * request, int free_request)
+void gftp_request_destroy (gftp_request * request, int free_request)
 {
+  DEBUG_PRINT_FUNC
   g_return_if_fail (request != NULL);
 
   gftp_disconnect (request);
 
-  if (request->destroy != NULL)
+  if (request->destroy != NULL) {
     request->destroy (request);
+  }
+  if (request->hostname)  g_free (request->hostname);
+  if (request->username)  g_free (request->username);
+  if (request->password)  g_free (request->password);
+  if (request->account)   g_free (request->account);
+  if (request->directory) g_free (request->directory);
 
-  if (request->hostname)
-    g_free (request->hostname);
-  if (request->username)
-    g_free (request->username);
-  if (request->password)
-    g_free (request->password);
-  if (request->account)
-    g_free (request->account);
-  if (request->directory)
-    g_free (request->directory);
-  if (request->last_ftp_response)
-    g_free (request->last_ftp_response);
-  if (request->protocol_data)
-    g_free (request->protocol_data);
-
-  if (request->remote_addr != NULL)
-    g_free (request->remote_addr);
+  if (request->last_ftp_response) g_free (request->last_ftp_response);
+  if (request->protocol_data)     g_free (request->protocol_data);
+  if (request->remote_addr)       g_free (request->remote_addr);
 
   if (request->local_options_vars != NULL)
     {
@@ -75,7 +71,6 @@ gftp_request_destroy (gftp_request * request, int free_request)
     {
       request->datafd = -1;
       request->cachefd = -1;
-      request->server_type = GFTP_DIRTYPE_OTHER;
     }
 }
 
@@ -87,6 +82,7 @@ void
 gftp_copy_param_options (gftp_request * dest_request,
                          gftp_request * src_request)
 {
+  DEBUG_PRINT_FUNC
   g_return_if_fail (dest_request != NULL);
   g_return_if_fail (src_request != NULL);
   g_return_if_fail (dest_request->protonum == src_request->protonum);
@@ -96,19 +92,15 @@ gftp_copy_param_options (gftp_request * dest_request,
 }
 
 
-void
-gftp_file_destroy (gftp_file * file, int free_it)
+void gftp_file_destroy (gftp_file * file, int free_it)
 {
+  //DEBUG_PRINT_FUNC
   g_return_if_fail (file != NULL);
 
-  if (file->file)
-    g_free (file->file);
-  if (file->user)
-    g_free (file->user);
-  if (file->group)
-    g_free (file->group);
-  if (file->destfile)
-    g_free (file->destfile);
+  if (file->file)     g_free (file->file);
+  if (file->user)     g_free (file->user);
+  if (file->group)    g_free (file->group);
+  if (file->destfile) g_free (file->destfile);
 
   if (free_it && file)
     g_free (file);
@@ -120,6 +112,7 @@ gftp_file_destroy (gftp_file * file, int free_it)
 int
 gftp_connect (gftp_request * request)
 {
+  DEBUG_PRINT_FUNC
   int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
@@ -137,6 +130,7 @@ gftp_connect (gftp_request * request)
 void
 gftp_disconnect (gftp_request * request)
 {
+  DEBUG_PRINT_FUNC
   g_return_if_fail (request != NULL);
 
   if (request->iconv_from_initialized)
@@ -168,6 +162,7 @@ off_t
 gftp_get_file (gftp_request * request, const char *filename,
                off_t startsize)
 {
+  DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
 
   request->cached = 0;
@@ -182,6 +177,7 @@ int
 gftp_put_file (gftp_request * request, const char *filename,
                off_t startsize, off_t totalsize)
 {
+  DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
 
   request->cached = 0;
@@ -197,6 +193,7 @@ gftp_transfer_file (gftp_request * fromreq, const char *fromfile,
                     off_t fromsize, gftp_request * toreq, const char *tofile,
                     off_t tosize)
 {
+  DEBUG_PRINT_FUNC
   /* Needed for systems that size(float) < size(void *) */
   union { intptr_t i; float f; } maxkbs;
   off_t size;
@@ -266,6 +263,7 @@ put_file:
 ssize_t 
 gftp_get_next_file_chunk (gftp_request * request, char *buf, size_t size)
 {
+  //DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (buf != NULL, GFTP_EFATAL);
 
@@ -279,6 +277,7 @@ gftp_get_next_file_chunk (gftp_request * request, char *buf, size_t size)
 ssize_t 
 gftp_put_next_file_chunk (gftp_request * request, char *buf, size_t size)
 {
+  //DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (buf != NULL, GFTP_EFATAL);
 
@@ -292,6 +291,7 @@ gftp_put_next_file_chunk (gftp_request * request, char *buf, size_t size)
 int
 gftp_end_transfer (gftp_request * request)
 {
+  DEBUG_PRINT_FUNC
   int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
@@ -322,6 +322,7 @@ gftp_end_transfer (gftp_request * request)
 int
 gftp_abort_transfer (gftp_request * request)
 {
+  DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
 
   if (request->abort_transfer == NULL)
@@ -336,6 +337,7 @@ int
 gftp_stat_filename (gftp_request * request, const char *filename, mode_t * mode,
                     off_t * filesize)
 {
+  DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (filename != NULL, GFTP_EFATAL);
 
@@ -349,6 +351,7 @@ gftp_stat_filename (gftp_request * request, const char *filename, mode_t * mode,
 int
 gftp_list_files (gftp_request * request)
 {
+  DEBUG_PRINT_FUNC
   char *remote_lc_time, *locret;
   int fd;
 
@@ -404,6 +407,7 @@ int
 gftp_get_next_file (gftp_request * request, const char *filespec,
                     gftp_file * fle)
 {
+  //DEBUG_PRINT_FUNC
   char *slashpos, *tmpfile, *utf8;
   size_t destlen;
   int fd, ret;
@@ -483,6 +487,7 @@ int
 gftp_parse_bookmark (gftp_request * request, gftp_request * local_request, 
                      const char * bookmark, int *refresh_local)
 {
+  DEBUG_PRINT_FUNC
   gftp_logging_func logging_function;
   gftp_bookmarks_var * tempentry;
   char *default_protocol;
@@ -495,6 +500,9 @@ gftp_parse_bookmark (gftp_request * request, gftp_request * local_request,
   logging_function = request->logging_function;
   gftp_request_destroy (request, 0);
   request->logging_function = logging_function;
+  if (!logging_function) {
+     fprintf (stderr, "gftp_parse_bookmark(): ERROR: logging_function is NULL! a crash is imminent!\n");
+  }
 
   if ((tempentry = g_hash_table_lookup (gftp_bookmarks_htable, 
                                         bookmark)) == NULL)
@@ -570,7 +578,7 @@ gftp_parse_bookmark (gftp_request * request, gftp_request * local_request,
         }
 
       if (gftp_protocols[i].url_prefix == NULL)
-        i = GFTP_FTP_NUM;
+        i = GFTP_PROTOCOL_FTP;
     }
 
   gftp_copy_local_options (&request->local_options_vars,
@@ -592,10 +600,12 @@ gftp_parse_bookmark (gftp_request * request, gftp_request * local_request,
 int
 gftp_parse_url (gftp_request * request, const char *url)
 {
+  DEBUG_PRINT_FUNC
   char *pos, *endpos, *default_protocol, *new_url;
   gftp_logging_func logging_function;
   const char *clear_pos;
-  int i, ret;
+  int i, ret, is_directory;
+  struct stat statbuf;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (url != NULL, GFTP_EFATAL);
@@ -643,7 +653,7 @@ gftp_parse_url (gftp_request * request, const char *url)
       gftp_lookup_request_option (request, "default_protocol", 
                                   &default_protocol);
 
-      i = GFTP_FTP_NUM;
+      i = GFTP_PROTOCOL_FTP;
       if (default_protocol != NULL && *default_protocol != '\0')
         {
           for (i = 0; gftp_protocols[i].url_prefix; i++)
@@ -673,8 +683,20 @@ gftp_parse_url (gftp_request * request, const char *url)
 
   if ((endpos = strchr (pos, '/')) != NULL)
     {
-      gftp_set_directory (request, endpos);
-      *endpos = '\0';
+      if (stat (endpos, &statbuf) == -1) {
+         is_directory = 0;
+      } else {
+         is_directory = S_ISDIR(statbuf.st_mode);
+      }
+      if (is_directory) {
+         gftp_set_directory (request, endpos);
+         *endpos = '\0';
+      } else {
+         if (request->directory) {
+            g_free (request->directory);
+         }
+         request->directory = g_strdup (endpos);
+      }
     }
 
   if (request->parse_url != NULL)
@@ -728,6 +750,7 @@ gftp_parse_url (gftp_request * request, const char *url)
 void
 gftp_set_hostname (gftp_request * request, const char *hostname)
 {
+  DEBUG_PRINT_FUNC
   g_return_if_fail (request != NULL);
   g_return_if_fail (hostname != NULL);
 
@@ -740,6 +763,7 @@ gftp_set_hostname (gftp_request * request, const char *hostname)
 void
 gftp_set_username (gftp_request * request, const char *username)
 {
+  DEBUG_PRINT_FUNC
   g_return_if_fail (request != NULL);
 
   if (request->username)
@@ -755,6 +779,7 @@ gftp_set_username (gftp_request * request, const char *username)
 void
 gftp_set_password (gftp_request * request, const char *password)
 {
+  DEBUG_PRINT_FUNC
   g_return_if_fail (request != NULL);
 
   if (request->password)
@@ -770,6 +795,7 @@ gftp_set_password (gftp_request * request, const char *password)
 void
 gftp_set_account (gftp_request * request, const char *account)
 {
+  DEBUG_PRINT_FUNC
   g_return_if_fail (request != NULL);
   g_return_if_fail (account != NULL);
 
@@ -782,10 +808,10 @@ gftp_set_account (gftp_request * request, const char *account)
 int
 gftp_set_directory (gftp_request * request, const char *directory)
 {
+  DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
 
-
   if (request->datafd <= 0 && !request->always_connected)
     {
       if (directory != request->directory)
@@ -805,8 +831,8 @@ gftp_set_directory (gftp_request * request, const char *directory)
 void
 gftp_set_port (gftp_request * request, unsigned int port)
 {
+  DEBUG_PRINT_FUNC
   g_return_if_fail (request != NULL);
-
   request->port = port;
 }
 
@@ -814,6 +840,7 @@ gftp_set_port (gftp_request * request, unsigned int port)
 int
 gftp_remove_directory (gftp_request * request, const char *directory)
 {
+  DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
 
   if (request->rmdir == NULL)
@@ -825,6 +852,7 @@ gftp_remove_directory (gftp_request * request, const char *directory)
 int
 gftp_remove_file (gftp_request * request, const char *file)
 {
+  DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
 
   if (request->rmfile == NULL)
@@ -836,6 +864,7 @@ gftp_remove_file (gftp_request * request, const char *file)
 int
 gftp_make_directory (gftp_request * request, const char *directory)
 {
+  DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
 
   if (request->mkdir == NULL)
@@ -849,6 +878,7 @@ int
 gftp_rename_file (gftp_request * request, const char *oldname,
                   const char *newname)
 {
+  DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
 
   if (request->rename == NULL)
@@ -861,6 +891,7 @@ gftp_rename_file (gftp_request * request, const char *oldname,
 int
 gftp_chmod (gftp_request * request, const char *file, mode_t mode)
 {
+  DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
 
   if (request->chmod == NULL)
@@ -874,6 +905,7 @@ gftp_chmod (gftp_request * request, const char *file, mode_t mode)
 int
 gftp_set_file_time (gftp_request * request, const char *file, time_t datetime)
 {
+  DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
 
   if (request->set_file_time == NULL)
@@ -885,6 +917,7 @@ gftp_set_file_time (gftp_request * request, const char *file, time_t datetime)
 int
 gftp_site_cmd (gftp_request * request, int specify_site, const char *command)
 {
+  DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
 
   if (request->site == NULL)
@@ -896,6 +929,7 @@ gftp_site_cmd (gftp_request * request, int specify_site, const char *command)
 off_t
 gftp_get_file_size (gftp_request * request, const char *filename)
 {
+  DEBUG_PRINT_FUNC
   g_return_val_if_fail (request != NULL, 0);
 
   if (request->get_file_size == NULL)
@@ -907,6 +941,7 @@ gftp_get_file_size (gftp_request * request, const char *filename)
 static GHashTable *
 gftp_gen_dir_hash (gftp_request * request, int *ret)
 {
+//  DEBUG_PRINT_FUNC
   GHashTable * dirhash;
   gftp_file * fle;
   off_t *newsize;
@@ -940,7 +975,7 @@ gftp_gen_dir_hash (gftp_request * request, int *ret)
 static void
 destroy_hash_ent (gpointer key, gpointer value, gpointer user_data)
 {
-
+//DEBUG_PRINT_FUNC
   g_free (key);
   g_free (value);
 }
@@ -960,6 +995,7 @@ gftp_destroy_dir_hash (GHashTable * dirhash)
 static GList *
 gftp_get_dir_listing (gftp_transfer * transfer, int getothdir, int *ret)
 {
+  DEBUG_PRINT_FUNC
   GHashTable * dirhash;
   GList * templist;
   gftp_file * fle;
@@ -1036,6 +1072,7 @@ _cleanup_get_all_subdirs (gftp_transfer * transfer, char *oldfromdir,
                           char *oldtodir,
                           void (*update_func) (gftp_transfer * transfer))
 {
+  DEBUG_PRINT_FUNC
   if (update_func != NULL)
     {
       transfer->numfiles = transfer->numdirs = -1;
@@ -1053,6 +1090,7 @@ _cleanup_get_all_subdirs (gftp_transfer * transfer, char *oldfromdir,
 static GList *
 _setup_current_directory_transfer (gftp_transfer * transfer, int *ret)
 {
+  DEBUG_PRINT_FUNC
   GHashTable * dirhash;
   char *pos, *newname;
   gftp_file * curfle;
@@ -1127,6 +1165,7 @@ static int
 _lookup_curfle_in_device_hash (gftp_request * request, gftp_file * curfle,
                                GHashTable * device_hash)
 {
+  DEBUG_PRINT_FUNC
   GHashTable * inode_hash;
 
   if (curfle->st_dev == 0 || curfle->st_ino == 0)
@@ -1164,6 +1203,7 @@ _lookup_curfle_in_device_hash (gftp_request * request, gftp_file * curfle,
 static void
 _free_inode_hash (gpointer key, gpointer value, gpointer user_data)
 {
+//DEBUG_PRINT_FUNC
   g_hash_table_destroy (value);
 }
 
@@ -1171,6 +1211,7 @@ _free_inode_hash (gpointer key, gpointer value, gpointer user_data)
 static void
 _free_device_hash (GHashTable * device_hash)
 {
+//DEBUG_PRINT_FUNC
   g_hash_table_foreach (device_hash, _free_inode_hash, NULL);
   g_hash_table_destroy (device_hash);
 }
@@ -1180,6 +1221,7 @@ int
 gftp_get_all_subdirs (gftp_transfer * transfer,
                       void (*update_func) (gftp_transfer * transfer))
 {
+  DEBUG_PRINT_FUNC
   GList * templist, * lastlist;
   char *oldfromdir, *oldtodir;
   GHashTable * device_hash;
@@ -1332,6 +1374,7 @@ gftp_get_all_subdirs (gftp_transfer * transfer,
 int
 gftp_set_config_options (gftp_request * request)
 {
+  DEBUG_PRINT_FUNC
   if (request->set_config_options != NULL)
     return (request->set_config_options (request));
   else
@@ -1342,6 +1385,7 @@ gftp_set_config_options (gftp_request * request)
 void
 print_file_list (GList * list)
 {
+  DEBUG_PRINT_FUNC
   gftp_file * tempfle;
   GList * templist;
   char *attribs;
@@ -1384,6 +1428,7 @@ print_file_list (GList * list)
 void
 gftp_swap_socks (gftp_request * dest, gftp_request * source)
 {
+  DEBUG_PRINT_FUNC
   g_return_if_fail (dest != NULL);
   g_return_if_fail (source != NULL);
   g_return_if_fail (dest->protonum == source->protonum);
@@ -1405,6 +1450,7 @@ gftp_swap_socks (gftp_request * dest, gftp_request * source)
 void
 gftp_calc_kbs (gftp_transfer * tdata, ssize_t num_read)
 {
+  DEBUG_PRINT_FUNC
   /* Needed for systems that size(float) < size(void *) */
   union { intptr_t i; float f; } maxkbs;
   unsigned long waitusecs;
@@ -1458,6 +1504,7 @@ gftp_calc_kbs (gftp_transfer * tdata, ssize_t num_read)
 static int
 _do_sleep (int sleep_time)
 {
+  DEBUG_PRINT_FUNC
   struct timeval tv;
 
   tv.tv_sec = sleep_time;
@@ -1470,6 +1517,7 @@ _do_sleep (int sleep_time)
 int
 gftp_get_transfer_status (gftp_transfer * tdata, ssize_t num_read)
 {
+  DEBUG_PRINT_FUNC
   intptr_t retries, sleep_time;
   gftp_file * tempfle;
   int ret1, ret2;
@@ -1585,6 +1633,7 @@ gftp_get_transfer_status (gftp_transfer * tdata, ssize_t num_read)
 int
 gftp_fd_open (gftp_request * request, const char *pathname, int flags, mode_t mode)
 {
+  DEBUG_PRINT_FUNC
   int fd;
 
   if (mode == 0)
@@ -1618,6 +1667,7 @@ gftp_fd_open (gftp_request * request, const char *pathname, int flags, mode_t mo
 void
 gftp_setup_startup_directory (gftp_request * request, const char *option_name)
 {
+  DEBUG_PRINT_FUNC
   char *startup_directory, *tempstr;
 
   gftp_lookup_request_option (request, option_name, &startup_directory);
@@ -1631,34 +1681,21 @@ gftp_setup_startup_directory (gftp_request * request, const char *option_name)
 }
 
 
-char *
-gftp_convert_attributes_from_mode_t (mode_t mode)
+char * gftp_convert_attributes_from_mode_t (mode_t mode)
 {
+  //DEBUG_PRINT_FUNC
   char *str;
 
   str = g_malloc0 (11UL);
   
   str[0] = '?';
-  if (S_ISREG (mode))
-    str[0] = '-';
-
-  if (S_ISLNK (mode))
-    str[0] = 'l';
-
-  if (S_ISBLK (mode))
-    str[0] = 'b';
-
-  if (S_ISCHR (mode))
-    str[0] = 'c';
-
-  if (S_ISFIFO (mode))
-    str[0] = 'p';
-
-  if (S_ISSOCK (mode))
-    str[0] = 's';
-
-  if (S_ISDIR (mode))
-    str[0] = 'd';
+  if (S_ISREG (mode))  str[0] = '-';
+  if (S_ISLNK (mode))  str[0] = 'l';
+  if (S_ISBLK (mode))  str[0] = 'b';
+  if (S_ISCHR (mode))  str[0] = 'c';
+  if (S_ISFIFO (mode)) str[0] = 'p';
+  if (S_ISSOCK (mode)) str[0] = 's';
+  if (S_ISDIR  (mode)) str[0] = 'd';
 
   str[1] = mode & S_IRUSR ? 'r' : '-';
   str[2] = mode & S_IWUSR ? 'w' : '-';
@@ -1700,23 +1737,17 @@ gftp_convert_attributes_from_mode_t (mode_t mode)
 }
 
 
-mode_t
-gftp_convert_attributes_to_mode_t (char *attribs)
+mode_t gftp_convert_attributes_to_mode_t (char *attribs)
 {
+  //DEBUG_PRINT_FUNC
   mode_t mode;
 
-  if (attribs[0] == 'd')
-    mode = S_IFDIR;
-  else if (attribs[0] == 'l')
-    mode = S_IFLNK;
-  else if (attribs[0] == 's')
-    mode = S_IFSOCK;
-  else if (attribs[0] == 'b')
-    mode = S_IFBLK;
-  else if (attribs[0] == 'c')
-    mode = S_IFCHR;
-  else
-    mode = S_IFREG;
+  if      (attribs[0] == 'd') mode = S_IFDIR;
+  else if (attribs[0] == 'l') mode = S_IFLNK;
+  else if (attribs[0] == 's') mode = S_IFSOCK;
+  else if (attribs[0] == 'b') mode = S_IFBLK;
+  else if (attribs[0] == 'c') mode = S_IFCHR;
+  else                        mode = S_IFREG;
 
   if (attribs[1] == 'r')
     mode |= S_IRUSR;
@@ -1741,8 +1772,7 @@ gftp_convert_attributes_to_mode_t (char *attribs)
     mode |= S_IROTH;
   if (attribs[8] == 'w')
     mode |= S_IWOTH;
-  if (attribs[9] == 'x' ||
-      attribs[9] == 's')
+  if (attribs[9] == 'x' || attribs[9] == 's')
     mode |= S_IXOTH;
   if (attribs[9] == 't' || attribs[9] == 'T')
     mode |= S_ISVTX;
@@ -1754,6 +1784,7 @@ gftp_convert_attributes_to_mode_t (char *attribs)
 unsigned int
 gftp_protocol_default_port (gftp_request * request)
 {
+  DEBUG_PRINT_FUNC
   struct servent serv_struct;
 
   if (r_getservbyname (gftp_protocols[request->protonum].url_prefix, "tcp",
