@@ -1,21 +1,25 @@
-/*****************************************************************************/
-/*  gftp-gtk.c - GTK+ port of gftp                                           */
-/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
-/*                                                                           */
-/*  This program is free software; you can redistribute it and/or modify     */
-/*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 2 of the License, or        */
-/*  (at your option) any later version.                                      */
-/*                                                                           */
-/*  This program is distributed in the hope that it will be useful,          */
-/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
-/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
-/*  GNU General Public License for more details.                             */
-/*                                                                           */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA      */
-/*****************************************************************************/
+/***********************************************************************************/
+/*  gftp-gtk.c - GTK+ port of gftp                                                 */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                        */
+/*                                                                                 */
+/*  Permission is hereby granted, free of charge, to any person obtaining a copy   */
+/*  of this software and associated documentation files (the "Software"), to deal  */
+/*  in the Software without restriction, including without limitation the rights   */
+/*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      */
+/*  copies of the Software, and to permit persons to whom the Software is          */
+/*  furnished to do so, subject to the following conditions:                       */
+/*                                                                                 */
+/*  The above copyright notice and this permission notice shall be included in all */
+/*  copies or substantial portions of the Software.                                */
+/*                                                                                 */
+/*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     */
+/*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       */
+/*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    */
+/*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         */
+/*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  */
+/*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  */
+/*  SOFTWARE.                                                                      */
+/***********************************************************************************/
 
 #include "gftp-gtk.h"
 #include "listbox.c"
@@ -25,7 +29,7 @@ static GtkWidget * local_frame, * remote_frame, * log_scroll, * transfer_scroll,
 
 GtkWindow *main_window;
 gftp_window_data window1, window2, *other_wdata, *current_wdata;
-GtkWidget * stop_btn, * hostedit, * useredit, * passedit, * portedit, * logwdw,
+GtkWidget * stop_btn, * hostedit, * useredit, * passedit, * portedit, * logwdw = NULL,
           * dlwdw, * toolbar_combo_protocol, * gftpui_command_widget, * download_left_arrow,
           * upload_right_arrow, * openurl_btn;
 GtkAdjustment * logwdw_vadj;
@@ -40,6 +44,7 @@ GtkUIManager * factory = NULL;
 pthread_mutex_t log_mutex = PTHREAD_MUTEX_INITIALIZER;
 pthread_t main_thread_id;
 GList * viewedit_processes = NULL;
+intptr_t gftp_gtk_colored_msgs = 0;
 
 static gboolean on_key_press_combo_toolbar(GtkWidget *widget, GdkEventKey *event, gpointer data);
 static void on_combo_protocol_change_cb (GtkComboBox *cb, gpointer data);
@@ -92,12 +97,19 @@ _gftp_exit (GtkWidget * widget, gpointer data)
                              &remember_last_directory);
   if (remember_last_directory)
     {
-      tempstr = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(window1.combo));
+      tempstr = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(window1.dir_combo));
       gftp_set_global_option ("local_startup_directory", tempstr);
 
-      tempstr = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(window2.combo));
+      tempstr = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(window2.dir_combo));
       gftp_set_global_option ("remote_startup_directory", tempstr);
     }
+  else
+    {
+      // problem: last_directory is always set, we must forget it
+      // until remember_last_directory=1 again
+      gftp_set_global_option ("local_startup_directory", "");
+      gftp_set_global_option ("remote_startup_directory", "");
+    }
 
   tempstr = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(toolbar_combo_protocol));
   gftp_set_global_option ("default_protocol", tempstr);
@@ -127,7 +139,8 @@ _gftp_try_close (GtkWidget * widget, GdkEvent * event, gpointer data)
     }
   else
     {
-      MakeYesNoDialog (_("Exit"), _("There are file transfers in progress.\nAre you sure you want to exit?"), _gftp_exit, NULL, NULL, NULL);
+      YesNoDialog (main_window, _("Exit"), _("There are file transfers in progress.\nAre you sure you want to exit?"),
+                   _gftp_exit, NULL, NULL, NULL);
       return (1);
     }
 }
@@ -147,8 +160,19 @@ _gftp_menu_exit (GtkWidget * widget, gpointer data)
     _gftp_exit (widget, data);
 }
 
-static void
-change_setting (GtkAction *action, GtkRadioAction *current)
+
+static void on_activate_colored_msgs_cb (GtkToggleAction * checkb)
+{
+   gboolean state = gtk_toggle_action_get_active (checkb);
+   gftp_set_global_option ("colored_msgs_gtk", GINT_TO_POINTER(state));
+   gftp_gtk_colored_msgs = state;
+   if (logwdw) {
+      ftp_log (gftp_logging_misc, NULL,
+               state ? _("-- Colors enabled\n") : _("-- Colors disabled\n"));
+   }
+}
+
+static void change_setting (GtkAction *action, GtkRadioAction *current)
 {
   gint menuitem = gtk_radio_action_get_current_value(current);
   switch (menuitem)
@@ -182,17 +206,9 @@ change_setting (GtkAction *action, GtkRadioAction *current)
 static void
 _gftpui_gtk_do_openurl (gftp_window_data * wdata, gftp_dialog_data * ddata)
 {
-  const char *tempstr;
-  char *buf;
-
-  tempstr = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
-  if (tempstr != NULL && *tempstr != '\0')
-    {
-      buf = g_strdup (tempstr);
-      destroy_dialog (ddata);
-      gftpui_common_cmd_open (wdata, wdata->request, NULL, NULL, buf);
-      g_free (buf);
-    }
+  if (ddata->entry_text[0] != '\0') {
+     gftpui_common_cmd_open (wdata, wdata->request, NULL, NULL, ddata->entry_text);
+  }
 }
 
 
@@ -202,10 +218,10 @@ openurl_dialog (gpointer data)
   gftp_window_data * wdata;
 
   wdata = data;
-  MakeEditDialog (_("Open Location"), _("Enter a URL to connect to"),
-                  NULL, 1, NULL, gftp_dialog_button_connect,
-                  _gftpui_gtk_do_openurl, wdata,
-                  NULL, NULL);
+  TextEntryDialog (NULL, _("Open Location"), _("Enter a URL to connect to"),
+                   NULL, 1, NULL, gftp_dialog_button_connect,
+                   _gftpui_gtk_do_openurl, wdata,
+                   NULL, NULL);
 }
 
 
@@ -240,196 +256,147 @@ gftp_gtk_refresh (gftp_window_data * wdata)
 }
 
 
-static void
-navi_up_directory(gftp_window_data * wdata)
+static void navi_up_directory (gftp_window_data * wdata)
 {
-  char *directory;
-  if(!gtk_widget_is_focus(window1.listbox) && !gtk_widget_is_focus(window2.listbox))
-  {
-     ftp_log (gftp_logging_error, NULL, _("Set the focus on a list window\n"));
+  char * dir;
+  if (!GFTP_IS_CONNECTED (wdata->request)) {
      return;
   }
-
-  if(gtk_widget_is_focus(window1.listbox)) {
-     wdata = &window1;
-  } else {
-     if (!GFTP_IS_CONNECTED (window2.request))
-         return;
-     wdata = &window2;
-  }
-
-  directory = gftp_build_path (wdata->request, wdata->request->directory,
-                               "..", NULL);
-  gftpui_run_chdir (wdata, directory);
-  g_free (directory);
+  dir = gftp_build_path (wdata->request, wdata->request->directory, "..", NULL);
+  gftpui_run_chdir (wdata, dir);
+  g_free (dir);
 }
 
 // ===
-static void
-on_local_openurl_dialog(void) {
+static void on_local_openurl_dialog (void) {
   openurl_dialog(&window1);
 }
-static void
-on_remote_openurl_dialog(void) {
+static void on_remote_openurl_dialog (void) {
   openurl_dialog(&window2);
 }
 
-static void
-on_local_gftpui_disconnect(void) {
+static void on_local_gftpui_disconnect (void) {
   gftpui_disconnect(&window1);
 }
-static void
-on_remote_gftpui_disconnect(void) {
+static void on_remote_gftpui_disconnect (void) {
   gftpui_disconnect(&window2);
 }
 
-static void
-on_local_change_filespec(void) {
+static void on_local_change_filespec (void) {
   change_filespec(&window1);
 }
-static void
-on_remote_change_filespec(void) {
+static void on_remote_change_filespec (void) {
   change_filespec(&window2);
 }
 
-static void
-on_local_show_selected(void) {
+static void on_local_show_selected (void) {
   show_selected(&window1);
 }
-static void
-on_remote_show_selected(void) {
+static void on_remote_show_selected (void) {
   show_selected(&window2);
 }
 
-static void
-on_local_navi_up_directory(void) {
+static void on_local_navi_up_directory (void) {
   navi_up_directory(&window1);
 }
-static void
-on_remote_navi_up_directory(void) {
+static void on_remote_navi_up_directory (void) {
   navi_up_directory(&window2);
 }
 
-static void
-on_local_selectall(void) {
+static void on_local_selectall (void) {
   window1.show_selected = 0;
   listbox_select_all(&window1);
 }
-static void
-on_remote_selectall(void) {
+static void on_remote_selectall (void) {
   window2.show_selected = 0;
   listbox_select_all(&window2);
 }
 
-static void
-on_local_selectallfiles(void) {
+static void on_local_selectallfiles (void) {
   window1.show_selected = 0;
   listbox_select_all_files(&window1);
 }
-static void
-on_remote_selectallfiles(void) {
+static void on_remote_selectallfiles (void) {
   window2.show_selected = 0;
   listbox_select_all_files(&window2);
 }
 
-static void
-on_local_deselectall(void) {
+static void on_local_deselectall (void) {
   window1.show_selected = 0;
   listbox_deselect_all(&window1);
 }
-static void
-on_remote_deselectall(void) {
+static void on_remote_deselectall (void) {
   window2.show_selected = 0;
   listbox_deselect_all(&window2);
 }
 
-static void
-on_local_save_directory_listing(void) {
+static void on_local_save_directory_listing (void) {
   save_directory_listing(&window1);
 }
-static void
-on_remote_save_directory_listing(void) {
+static void on_remote_save_directory_listing (void) {
   save_directory_listing(&window2);
 }
 
-static void
-on_local_gftpui_site_dialog(void) {
+static void on_local_gftpui_site_dialog (void) {
   gftpui_site_dialog(&window1);
 }
-static void
-on_remote_gftpui_site_dialog(void) {
+static void on_remote_gftpui_site_dialog (void) {
   gftpui_site_dialog(&window2);
 }
 
-static void
-on_local_gftpui_chdir_dialog(void) {
+static void on_local_gftpui_chdir_dialog (void) {
   gftpui_chdir_dialog(&window1);
 }
-static void
-on_remote_gftpui_chdir_dialog(void) {
+static void on_remote_gftpui_chdir_dialog (void) {
   gftpui_chdir_dialog(&window2);
 }
 
-static void
-on_local_chmod_dialog(void) {
+static void on_local_chmod_dialog (void) {
   chmod_dialog(&window1);
 }
-static void
-on_remote_chmod_dialog(void) {
+static void on_remote_chmod_dialog (void) {
   chmod_dialog(&window2);
 }
 
-static void
-on_local_gftpui_mkdir_dialog(void) {
+static void on_local_gftpui_mkdir_dialog (void) {
   gftpui_mkdir_dialog(&window1);
 }
-static void
-on_remote_gftpui_mkdir_dialog(void) {
+static void on_remote_gftpui_mkdir_dialog (void) {
   gftpui_mkdir_dialog(&window2);
 }
 
-static void
-on_local_gftpui_rename_dialog(void) {
+static void on_local_gftpui_rename_dialog (void) {
   gftpui_rename_dialog(&window1);
 }
-static void
-on_remote_gftpui_rename_dialog(void) {
+static void on_remote_gftpui_rename_dialog (void) {
   gftpui_rename_dialog(&window2);
 }
 
-static void
-on_local_delete_dialog(void) {
+static void on_local_delete_dialog (void) {
   delete_dialog(&window1);
 }
-static void
-on_remote_delete_dialog(void) {
+static void on_remote_delete_dialog (void) {
   delete_dialog(&window2);
 }
 
-static void
-on_local_edit_dialog(void) {
+static void on_local_edit_dialog(void) {
   edit_dialog(&window1);
 }
-static void
-on_remote_edit_dialog(void) {
+static void on_remote_edit_dialog(void) {
   edit_dialog(&window2);
 }
 
-static void
-on_local_view_dialog(void) {
+static void on_local_view_dialog (void) {
   view_dialog(&window1);
 }
-static void
-on_remote_view_dialog(void) {
+static void on_remote_view_dialog (void) {
   view_dialog(&window2);
 }
 
-static void
-on_local_gftp_gtk_refresh(void) {
+static void on_local_gftp_gtk_refresh (void) {
   gftp_gtk_refresh(&window1);
 }
-static void
-on_remote_gftp_gtk_refresh(void) {
+static void on_remote_gftp_gtk_refresh (void) {
   gftp_gtk_refresh(&window2);
 }
 // ===
@@ -440,11 +407,14 @@ CreateMenus (GtkWidget * parent)
   GtkAccelGroup * accel_group;
   intptr_t ascii_transfers;
   int default_ascii;
+  GtkWidget * colormsg_chkbox;
+
+  gftp_lookup_global_option ("colored_msgs_gtk", &gftp_gtk_colored_msgs);
 
   static const GtkActionEntry menu_items[] =
   {
     //  name                    stock_id               "label"                  accel             tooltip  callback
-    { "FTPMenu",              NULL,                  N_("_FTP"),              NULL,                NULL, NULL                   },
+    { "FTPMenu",              NULL,                  N_("g_FTP"),             NULL,                NULL, NULL                   },
     { "FTPPreferences",       "gtk-preferences",     N_("_Preferences..."),   NULL,                NULL, G_CALLBACK(options_dialog) },
     { "FTPQuit",              "gtk-quit",            N_("_Quit"),             "<control>Q",        NULL, G_CALLBACK(_gftp_menu_exit)  },
 
@@ -453,7 +423,7 @@ CreateMenus (GtkWidget * parent)
     { "LocalDisconnect",      "gtk-close",           N_("D_isconnect"),       "<control><shift>I", NULL, G_CALLBACK(on_local_gftpui_disconnect) },
     { "LocalChangeFilespec",  NULL,                  N_("Change _Filespec"),  "<control><shift>F", NULL, G_CALLBACK(on_local_change_filespec) },
     { "LocalShowSelected",    NULL,                  N_("_Show selected"),    NULL,                NULL, G_CALLBACK(on_local_show_selected) },
-    { "LocalNavigateUp",      NULL,                  N_("Navigate _Up"),      "<alt>Up",           NULL, G_CALLBACK(on_local_navi_up_directory) },
+    { "LocalNavigateUp",      NULL,                  N_("Navigate _Up"),      "<ctrl><alt>Up",     NULL, G_CALLBACK(on_local_navi_up_directory) },
     { "LocalSelectAll",       NULL,                  N_("Select _All"),       "<control><shift>A", NULL, G_CALLBACK(on_local_selectall) },
     { "LocalSelectAllFiles",  NULL,                  N_("Select All Files"),  NULL,                NULL, G_CALLBACK(on_local_selectallfiles) },
     { "LocalDeselectAll",     NULL,                  N_("Deselect All"),      NULL,                NULL, G_CALLBACK(on_local_deselectall) },
@@ -515,6 +485,10 @@ CreateMenus (GtkWidget * parent)
     { "HelpAbout",           "gtk-about",            N_("_About"),            NULL,                NULL, G_CALLBACK(about_dialog) },
   };
 
+  static const GtkToggleActionEntry menu_check_colormsg[] = {
+    { "FTPColorMsg", NULL, N_("Colored messages"), "<control><alt>C", NULL,
+      G_CALLBACK (on_activate_colored_msgs_cb), FALSE }, //FALSE: need a constant element
+  };
 
   static const GtkRadioActionEntry menu_radio_window[] = {
     { "FTPWindow1", NULL, N_("Window _1"), "<control>1", NULL, GFTP_MENU_ITEM_WIN1 },
@@ -531,6 +505,8 @@ CreateMenus (GtkWidget * parent)
   <ui> \
     <menubar name='M'> \
       <menu action='FTPMenu'> \
+        <menuitem action='FTPColorMsg'/> \
+        <separator/> \
         <menuitem action='FTPWindow1'/> \
         <menuitem action='FTPWindow2'/> \
         <separator/> \
@@ -679,15 +655,24 @@ CreateMenus (GtkWidget * parent)
   factory = gtk_ui_manager_new();
   
   GtkActionGroup *actions = gtk_action_group_new("Actions");
-  
+
+  // must set translate_func or domain before adding actions
+  gtk_action_group_set_translation_domain (actions, "gftp");
+
   gtk_action_group_add_actions(actions, menu_items, nmenu_items, NULL);
 
   current_wdata = &window2;
+
+  // colored msgs
+  gtk_action_group_add_toggle_actions (actions, menu_check_colormsg,
+                                       G_N_ELEMENTS (menu_check_colormsg), 0);
+  // Windows 1/2
   gtk_action_group_add_radio_actions(actions, menu_radio_window,
                                G_N_ELEMENTS (menu_radio_window),
                                GFTP_MENU_ITEM_WIN2,
                                G_CALLBACK(change_setting),
                                0);
+  // ascii binary
   gftp_lookup_global_option ("ascii_transfers", &ascii_transfers);
   if (ascii_transfers)
     default_ascii = GFTP_MENU_ITEM_ASCII;
@@ -714,6 +699,12 @@ CreateMenus (GtkWidget * parent)
 
   gtk_window_add_accel_group (GTK_WINDOW (parent), accel_group);
 
+  // the menu is now ready. enable/disable colored msgs
+  colormsg_chkbox = gtk_ui_manager_get_widget (factory, "/M/FTPMenu/FTPColorMsg");
+  if (colormsg_chkbox) {
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (colormsg_chkbox), gftp_gtk_colored_msgs);
+  }
+
   return (menu);
 }
 
@@ -728,9 +719,8 @@ CreateConnectToolbar (GtkWidget * parent)
   };
   GtkWidget *toolbar, *box, *tempwid;
   gftp_config_list_vars * tmplistvar;
-  GtkWidget *combo_entry;
+  GtkWidget *dir_combo_entry;
   char *default_protocol, *tempstr;
-  int i, j, num;
 
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
   gtk_container_set_border_width (GTK_CONTAINER (box), 5);
@@ -767,13 +757,13 @@ CreateConnectToolbar (GtkWidget * parent)
   glist_to_combobox (tmplistvar->list, hostedit);
 
   gftp_lookup_global_option ("host_value", &tempstr);
-  combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(hostedit)));
-  g_signal_connect (G_OBJECT(combo_entry), "key_press_event",
+  dir_combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(hostedit)));
+  g_signal_connect (G_OBJECT(dir_combo_entry), "key_press_event",
                    G_CALLBACK (on_key_press_combo_toolbar), NULL);
-  g_signal_connect (G_OBJECT(combo_entry), "key_release_event",
+  g_signal_connect (G_OBJECT(dir_combo_entry), "key_release_event",
                    G_CALLBACK (on_key_press_combo_toolbar), NULL);
 
-  gtk_entry_set_text (GTK_ENTRY (combo_entry), tempstr);
+  gtk_entry_set_text (GTK_ENTRY (dir_combo_entry), tempstr);
   gtk_box_pack_start (GTK_BOX (box), hostedit, TRUE, TRUE, 0);
 
   tempwid = gtk_label_new (_("Port:"));
@@ -787,13 +777,13 @@ CreateConnectToolbar (GtkWidget * parent)
 
   gftp_lookup_global_option ("port_value", &tempstr);
 
-  combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(portedit)));
-  g_signal_connect (G_OBJECT(combo_entry), "key_press_event",
+  dir_combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(portedit)));
+  g_signal_connect (G_OBJECT(dir_combo_entry), "key_press_event",
                    G_CALLBACK (on_key_press_combo_toolbar), NULL);
-  g_signal_connect (G_OBJECT(combo_entry), "key_release_event",
+  g_signal_connect (G_OBJECT(dir_combo_entry), "key_release_event",
                    G_CALLBACK (on_key_press_combo_toolbar), NULL);
 
-  gtk_entry_set_text (GTK_ENTRY (combo_entry), tempstr);
+  gtk_entry_set_text (GTK_ENTRY (dir_combo_entry), tempstr);
   gtk_box_pack_start (GTK_BOX (box), portedit, FALSE, FALSE, 0);
 
   tempwid = gtk_label_new_with_mnemonic (_("_User:"));
@@ -807,13 +797,13 @@ CreateConnectToolbar (GtkWidget * parent)
 
   gftp_lookup_global_option ("user_value", &tempstr);
 
-  combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(useredit)));
-  g_signal_connect (G_OBJECT(combo_entry), "key_press_event",
+  dir_combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(useredit)));
+  g_signal_connect (G_OBJECT(dir_combo_entry), "key_press_event",
                    G_CALLBACK (on_key_press_combo_toolbar), NULL);
-  g_signal_connect (G_OBJECT(combo_entry), "key_release_event",
+  g_signal_connect (G_OBJECT(dir_combo_entry), "key_release_event",
                    G_CALLBACK (on_key_press_combo_toolbar), NULL);
 
-  gtk_entry_set_text (GTK_ENTRY (combo_entry), tempstr);
+  gtk_entry_set_text (GTK_ENTRY (dir_combo_entry), tempstr);
   gtk_box_pack_start (GTK_BOX (box), useredit, TRUE, TRUE, 0);
 
   tempwid = gtk_label_new (_("Pass:"));
@@ -838,20 +828,9 @@ CreateConnectToolbar (GtkWidget * parent)
                     "changed",
                     G_CALLBACK (on_combo_protocol_change_cb),
                     NULL);
-  num = 0;
-  j = 0;
+
   gftp_lookup_global_option ("default_protocol", &default_protocol);
-  for (i = 0; gftp_protocols[i].name; i++)
-    {
-      if (!gftp_protocols[i].shown)
-        continue;
-      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (toolbar_combo_protocol),
-                                 gftp_protocols[i].name);
-      if (default_protocol && strcmp (gftp_protocols[i].name, default_protocol) == 0)
-        num = j;
-      j++;
-    }
-  gtk_combo_box_set_active(GTK_COMBO_BOX(toolbar_combo_protocol), num);
+  populate_combo_and_select_protocol (toolbar_combo_protocol, default_protocol);
 
   //tempwid = gtk_image_new_from_icon_name ("gtk-stop", GTK_ICON_SIZE_SMALL_TOOLBAR);
   tempwid = gtk_image_new_from_stock ("gtk-stop", GTK_ICON_SIZE_SMALL_TOOLBAR);
@@ -873,12 +852,17 @@ CreateCommandToolbar (void)
 {
   GtkWidget * box, * tempwid;
 
-  gftpui_command_toolbar = gtk_handle_box_new ();
-
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
-  gtk_container_add (GTK_CONTAINER (gftpui_command_toolbar), box);
   gtk_container_set_border_width (GTK_CONTAINER (box), 5);
 
+#if GTK_MAJOR_VERSION == 2
+  // GtkHandleBox is broken in GTK3 - macOS
+  gftpui_command_toolbar = gtk_handle_box_new ();
+  gtk_container_add (GTK_CONTAINER (gftpui_command_toolbar), box);
+#else
+  gftpui_command_toolbar = box;
+#endif
+
   tempwid = gtk_label_new (_("Command: "));
   gtk_box_pack_start (GTK_BOX (box), tempwid, FALSE, FALSE, 0);
 
@@ -916,7 +900,7 @@ on_listbox_right_click_cb (GtkWidget *widget, GdkEventButton *event, gpointer da
   gftp_window_data * wdata = data;
 
   if (event->button == 3) {
-    if (strcmp (gftp_protocols[wdata->request->protonum].name, "Local") == 0) {
+    if (wdata->request->protonum == GFTP_PROTOCOL_LOCALFS) {
       GtkWidget* menu = gtk_ui_manager_get_widget(factory, "/LocalPopupMenu");
       gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, event->time);
     }
@@ -946,23 +930,20 @@ on_listbox_row_activated_cb (GtkTreeView *tree_view,    GtkTreePath *path,
   // list directory or download file
 
   gftp_file * tempfle = (gftp_file *) listbox_get_selected_files (wdata, 1);
+  if (!tempfle) {
+     fprintf (stderr, "* on_listbox_row_activated_cb(): ERROR\n"); // debug
+     return;
+  }
 
   char      *directory;
   int success = 0;
 
   if (S_ISLNK (tempfle->st_mode) || S_ISDIR (tempfle->st_mode))
   {
-      if(tempfle->st_mode & S_IXUSR)
-      {
-          directory = gftp_build_path (wdata->request, wdata->request->directory,
+      directory = gftp_build_path (wdata->request, wdata->request->directory,
                                    tempfle->file, NULL);
-          success = gftpui_run_chdir (wdata, directory);
-          g_free (directory);
-      }else{
-          ftp_log (gftp_logging_error, NULL,
-               _("Directory %s is not listable\n"),
-               tempfle->file);
-      }
+      success = gftpui_run_chdir (wdata, directory);
+      g_free (directory);
   }
 
   if (tempfle && !S_ISDIR (tempfle->st_mode) && !success)
@@ -995,7 +976,7 @@ CreateFTPWindow (gftp_window_data * wdata)
   char tempstr[50], *startup_directory;
   GtkWidget *box, *scroll_list, *parent;
   intptr_t listbox_file_height, colwidth;
-  GtkWidget *combo_entry;
+  GtkWidget *dir_combo_entry;
 
   wdata->request = gftp_request_new ();
   gftp_gtk_init_request (wdata);
@@ -1013,25 +994,25 @@ CreateFTPWindow (gftp_window_data * wdata)
   gtk_container_set_border_width (GTK_CONTAINER (box), 5);
   gtk_container_add (GTK_CONTAINER (parent), box);
 
-  wdata->combo = gtk_combo_box_text_new_with_entry ();
-  gtk_box_pack_start (GTK_BOX (box), wdata->combo, FALSE, FALSE, 0);
+  wdata->dir_combo = gtk_combo_box_text_new_with_entry ();
+  gtk_box_pack_start (GTK_BOX (box), wdata->dir_combo, FALSE, FALSE, 0);
 
-  combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(wdata->combo)));
-  g_signal_connect (G_OBJECT(combo_entry), "key_press_event",
-                   G_CALLBACK (chdir_edit), NULL);
-  g_signal_connect (G_OBJECT(combo_entry), "key_release_event",
-                   G_CALLBACK (chdir_edit), wdata);
+  dir_combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(wdata->dir_combo)));
+  g_signal_connect (G_OBJECT(dir_combo_entry), "key_press_event",
+                   G_CALLBACK (dir_combo_keycb), NULL);
+  g_signal_connect (G_OBJECT(dir_combo_entry), "key_release_event",
+                   G_CALLBACK (dir_combo_keycb), wdata);
 
-  glist_to_combobox (*wdata->history, wdata->combo);
+  glist_to_combobox (*wdata->history, wdata->dir_combo);
 
   g_snprintf (tempstr, sizeof (tempstr), "%s_startup_directory",
               wdata->prefix_col_str);
   gftp_lookup_global_option (tempstr, &startup_directory);
-  gtk_entry_set_text (GTK_ENTRY (combo_entry), startup_directory);
+  gtk_entry_set_text (GTK_ENTRY (dir_combo_entry), startup_directory);
 
-  wdata->hoststxt = gtk_label_new (NULL);
-  gtkcompat_widget_set_halign_left (wdata->hoststxt);
-  gtk_box_pack_start (GTK_BOX (box), wdata->hoststxt, FALSE, FALSE, 0);
+  wdata->dirinfo_label = gtk_label_new (NULL);
+  gtkcompat_widget_set_halign_left (wdata->dirinfo_label);
+  gtk_box_pack_start (GTK_BOX (box), wdata->dirinfo_label, FALSE, FALSE, 0);
 
   scroll_list = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll_list),
@@ -1330,7 +1311,7 @@ toolbar_hostedit(void)
       gftp_parse_url (current_wdata->request, 
                       current_wdata->request->hostname) == 0)
     {
-      ftp_connect (current_wdata, current_wdata->request);
+      gftp_gtk_connect (current_wdata, current_wdata->request);
       return;
     }
  
@@ -1352,14 +1333,15 @@ toolbar_hostedit(void)
   gftp_set_password (current_wdata->request,
 		     gtk_entry_get_text (GTK_ENTRY (passedit)));
 
-  gftp_set_directory (current_wdata->request, gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(current_wdata->combo)) );
+  gftp_set_directory (current_wdata->request,
+                      gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(current_wdata->dir_combo)));
   if (current_wdata->request->directory != NULL)
     alltrim (current_wdata->request->directory);
 
-  add_history (current_wdata->combo, current_wdata->history, 
+  add_history (current_wdata->dir_combo, current_wdata->history, 
                current_wdata->histlen, current_wdata->request->directory);
 
-  ftp_connect (current_wdata, current_wdata->request);
+  gftp_gtk_connect (current_wdata, current_wdata->request);
 }
 
 void
@@ -1423,40 +1405,38 @@ gftp_gtk_config_file_write_color (gftp_config_vars * cv, char *buf,
 static void
 _setup_window1 ()
 {
-  if (gftp_protocols[GFTP_LOCAL_NUM].init (window1.request) == 0)
+  if (gftp_protocols[GFTP_PROTOCOL_LOCALFS].init (window1.request) == 0)
     {
       gftp_setup_startup_directory (window1.request,
                                     "local_startup_directory");
       gftp_connect (window1.request);
-      ftp_list_files (&window1);
+       gftp_gtk_list_files (&window1);
     }
 }
 
 
-static void
-_setup_window2 (int argc, char **argv)
+static void _setup_window2 (int argc, char **argv)
 {
   intptr_t connect_to_remote_on_startup;
-
+  char * rhost;
   gftp_lookup_request_option (window2.request, "connect_to_remote_on_startup",
                               &connect_to_remote_on_startup);
 
   if (argc == 2 && strncmp (argv[1], "--", 2) != 0)
     {
       if (gftp_parse_url (window2.request, argv[1]) == 0)
-        ftp_connect (&window2, window2.request);
+        gftp_gtk_connect (&window2, window2.request);
       else
         gftp_usage ();
     }
   else if (connect_to_remote_on_startup)
     {
-      if (gftp_protocols[_get_selected_protocol ()].init (current_wdata->request) == 0)
-        {
-          gftp_setup_startup_directory (window2.request,
-                                        "remote_startup_directory");
-          gftp_connect (window2.request);
-          ftp_list_files (&window2);
-        }
+       rhost = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT (hostedit));
+       if (rhost && *rhost) {
+          ftp_log (gftp_logging_misc, NULL, _("** Connect to remote server on startup is enabled\n"));
+          gftp_setup_startup_directory (window2.request, "remote_startup_directory");
+          tb_openurl_dialog (&window2);
+       }
     }
   else
     {
