@@ -4,7 +4,7 @@
  * For more information, please refer to <https://unlicense.org>
  */
 
-/** 2020-10-11 **/
+/** 2021-03-01 **/
 
 /*
  * gtkcompat.h, GTK2+ compatibility layer
@@ -26,6 +26,24 @@ special defines:
 	gtkcompat_widget_set_halign_right  (w)
 */
 
+/*
+GTKCOMPAT_DRAW_SIGNAL (gtk3="draw", gtk2="expose_event")
+---------------------
+ g_signal_connect (w, GTKCOMPAT_DRAW_SIGNAL, G_CALLBACK (w_draw_cb), NULL);
+ gboolean w_draw_cb (GtkWidget *w, gpointer compat, gpointer user_data)
+ {
+ #if GTK_CHECK_VERSION (3, 0, 0)
+    cairo_t * cr = (cairo_t *) compat;
+ #else // gtk2
+    //GdkEventExpose * event = (GdkEventExpose *) compat;
+    cairo_t * cr = gdk_cairo_create (gtk_widget_get_window (w));
+ #endif
+ #if GTK_MAJOR_VERSION == 2
+    cairo_destroy (cr);
+ #endif
+ }
+*/
+
 #ifndef __GTKCOMPAT_H
 #define __GTKCOMPAT_H
 
@@ -58,6 +76,16 @@ extern "C"
 #endif
 
 
+// GLIB < 2.40
+#if ! GLIB_CHECK_VERSION (2, 40, 0)
+#define g_key_file_save_to_file(kfile,filename,error) { \
+   char * data = g_key_file_to_data (kfile, NULL, error); \
+   g_file_set_contents (filename, data, -1, error); \
+   g_free (data); \
+}
+#endif
+
+
 // GLIB < 2.32
 #if ! GLIB_CHECK_VERSION (2, 32, 0)
 #define G_SOURCE_REMOVE   FALSE
@@ -104,7 +132,7 @@ extern "C"
 
 // GLIB < 2.28
 #if ! GLIB_CHECK_VERSION (2, 28, 0)
-#define g_list_free_full(list, free_func) {\
+#define g_list_free_full(list,free_func) {\
      g_list_foreach (list, (GFunc) free_func, NULL);\
      g_list_free (list);\
 }
@@ -127,53 +155,56 @@ extern "C"
 
 
 
-
 /* ================================================== */
 /*                       GTK 3                        */
 /* ================================================== */
 
-
 // GTK >= 3.0 -- applies to GTK3, GTK4...
-#if GTK_CHECK_VERSION (3, 0, 0)
+#if GTK_MAJOR_VERSION >= 3
+#define GTKCOMPAT_DRAW_SIGNAL "draw"
 #define gtkcompat_widget_set_halign_left(w)   gtk_widget_set_halign(GTK_WIDGET(w), GTK_ALIGN_START)
 #define gtkcompat_widget_set_halign_center(w) gtk_widget_set_halign(GTK_WIDGET(w), GTK_ALIGN_CENTER)
 #define gtkcompat_widget_set_halign_right(w)  gtk_widget_set_halign(GTK_WIDGET(w), GTK_ALIGN_END)
-/* some compatible deprecated GTK2 functions... */
-#define gtk_hbox_new(homogenous,spacing) gtk_box_new(GTK_ORIENTATION_HORIZONTAL,spacing)
-#define gtk_vbox_new(homogenous,spacing) gtk_box_new(GTK_ORIENTATION_VERTICAL,spacing)
-#define gtk_hscale_new(adjustment) gtk_scale_new(GTK_ORIENTATION_HORIZONTAL,adjustment)
-#define gtk_vscale_new(adjustment) gtk_scale_new(GTK_ORIENTATION_VERTICAL,adjustment)
-#define gtk_hscale_new_with_range(min,max,step) gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL,min,max,step)
-#define gtk_vscale_new_with_range(min,max,step) gtk_scale_new_with_range(GTK_ORIENTATION_VERTICAL,min,max,step)
-#define gtk_hseparator_new() gtk_separator_new(GTK_ORIENTATION_HORIZONTAL)
-#define gtk_vseparator_new() gtk_separator_new(GTK_ORIENTATION_VERTICAL)
-#define gtk_hscrollbar_new(adj) gtk_scrollbar_new(GTK_ORIENTATION_HORIZONTAL,adj)
-#define gtk_vscrollbar_new(adj) gtk_scrollbar_new(GTK_ORIENTATION_VERTICAL,adj)
-#define gtk_hpaned_new() gtk_paned_new(GTK_ORIENTATION_HORIZONTAL)
-#define gtk_vpaned_new() gtk_paned_new(GTK_ORIENTATION_VERTICAL)
 #endif
 
 
 // GTK < 3.12
 #if ! GTK_CHECK_VERSION (3, 12, 0)
+#define gtk_application_set_accels_for_action(app,name,accels) \
+          gtk_application_add_accelerator(app,accels[0],name,NULL)
 #define gtk_widget_set_margin_start(widget,margin) gtk_widget_set_margin_left(widget,margin)
 #define gtk_widget_set_margin_end(widget,margin)   gtk_widget_set_margin_right(widget,margin)
 #endif
 
 
+// GTK < 3.10
+#if GTK_CHECK_VERSION (3, 10, 0)
+// gdk_window_create_similar_image_surface() was removed in gtk4
+//                  only use gdk_window_create_similar_surface()
+#endif
+
+
+// GTK < 3.8
+#if ! GTK_CHECK_VERSION (3, 8, 0)
+#define gtk_widget_set_opacity(w,o) gtk_window_set_opacity(GTK_WINDOW(w),o)
+#define gtk_widget_get_opacity(w)  (gtk_window_get_opacity(GTK_WINDOW(w))
+#endif
+
+
 // GTK < 3.4
 #if ! GTK_CHECK_VERSION (3, 4, 0)
 #define gtk_application_window_new(app) gtk_window_new(GTK_WINDOW_TOPLEVEL)
 #endif
 
 
+
 /* ================================================== */
 /*                       GTK 2                        */
 /* ================================================== */
 
-
-// GTK < 3.0
-#if ! GTK_CHECK_VERSION (3, 0, 0)
+// define some GTK3+ functions
+#if GTK_MAJOR_VERSION <= 2
+#define GTKCOMPAT_DRAW_SIGNAL "expose_event"
 #define gtk_box_new(ori,spacing) \
   ((ori == GTK_ORIENTATION_HORIZONTAL) ? gtk_hbox_new(FALSE,spacing) \
                                        : gtk_vbox_new(FALSE,spacing))
@@ -197,6 +228,9 @@ extern "C"
                                        : gtk_vpaned_new())
 #define gtk_widget_get_allocated_height(widget) (GTK_WIDGET(widget)->allocation.height )
 #define gtk_widget_get_allocated_width(widget)  (GTK_WIDGET(widget)->allocation.width  )
+#define gtk_combo_box_text_remove_all(cmb) { \
+   gtk_list_store_clear (GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (cmb)))); \
+}
 #define gtk_tree_model_iter_previous(model,iter) ({ \
    GtkTreePath * path = gtk_tree_model_get_path (model, iter); \
    gboolean valid = gtk_tree_path_prev (path); \
@@ -204,6 +238,7 @@ extern "C"
    gtk_tree_path_free (path); \
    valid; \
 })
+#define gtk_widget_override_font(w,f) gtk_widget_modify_font(w,f)
 #define gtkcompat_widget_set_halign_left(w)   gtk_misc_set_alignment(GTK_MISC(w), 0.0, 0.5)
 #define gtkcompat_widget_set_halign_center(w) gtk_misc_set_alignment(GTK_MISC(w), 0.5, 0.5)
 #define gtkcompat_widget_set_halign_right(w)  gtk_misc_set_alignment(GTK_MISC(w), 1.0, 0.5)
@@ -229,14 +264,19 @@ typedef struct _GtkComboBoxPrivate GtkComboBoxTextPrivate;
 #define gtk_combo_box_text_new() gtk_combo_box_new_text()
 #define gtk_combo_box_text_new_with_entry()	gtk_combo_box_entry_new_text()
 #define gtk_combo_box_text_append_text(combo,text) gtk_combo_box_append_text(combo,text)
-#define gtk_combo_box_text_insert_text(combox,pos,text) gtk_combo_box_insert_text(combox,pos,text)
+#define gtk_combo_box_text_insert_text(combo,pos,text) gtk_combo_box_insert_text(combo,pos,text)
 #define gtk_combo_box_text_prepend_text(combo,text) gtk_combo_box_prepend_text(combo,text)
 #define gtk_combo_box_text_remove(combo,pos) gtk_combo_box_remove_text(combo,pos)
-#define gtk_combo_box_text_get_active_text(combo) gtk_combo_box_get_active_text(combo)
+#define gtk_combo_box_text_get_active_text(combo) (gtk_combo_box_get_active_text(combo))
+#define gtk_combo_box_get_has_entry(combo) (0)
+#define gtk_combo_box_set_entry_text_column(combo,cl)
+#define gtk_combo_box_get_entry_text_column(combo) (0)
 #define gtk_range_get_round_digits(range) (GTK_RANGE(range)->round_digits)
 //#define gdk_window_get_visual(w)  (gdk_drawable_get_visual(GDK_DRAWABLE(w)))
 #define gdk_window_get_screen(w)  (gdk_drawable_get_screen(GDK_DRAWABLE(w)))
 #define gdk_window_get_display(w) (gdk_drawable_get_display(GDK_DRAWABLE(w)))
+#define gtk_notebook_get_group_name gtk_notebook_get_group
+#define gtk_notebook_set_group_name gtk_notebook_set_group
 #endif
 
 
@@ -254,6 +294,15 @@ typedef struct _GtkComboBoxPrivate GtkComboBoxTextPrivate;
 #define gdk_visual_get_byte_order(visual) (GDK_VISUAL(visual)->byte_order)
 #define gdk_visual_get_colormap_size(visual) (GDK_VISUAL(visual)->colormap_size)
 #define gdk_visual_get_bits_per_rgb(visual) (GDK_VISUAL(visual)->bits_per_rgb)
+#define gtk_icon_view_get_item_orientation gtk_icon_view_get_orientation
+#define gtk_icon_view_set_item_orientation gtk_icon_view_set_orientation
+#define gdk_window_create_similar_surface(gdksurf,content,width,height) ({ \
+   cairo_t * wcr = gdk_cairo_create (gdksurf); \
+   cairo_surface_t * window_surface = cairo_get_target (wcr); \
+   cairo_surface_t * out_s = cairo_surface_create_similar (window_surface, content, width, height); \
+   cairo_destroy (wcr); \
+   out_s; \
+})
 #endif
 
 
@@ -262,11 +311,15 @@ typedef struct _GtkComboBoxPrivate GtkComboBoxTextPrivate;
 #define gtk_widget_get_mapped(wid) ((GTK_WIDGET_FLAGS (wid) & GTK_MAPPED) != 0)
 #define gtk_widget_get_realized(wid) ((GTK_WIDGET_FLAGS (wid) & GTK_REALIZED) != 0)
 #define gtk_window_get_window_type(window) (GTK_WINDOW(window)->type)
-#define gtk_widget_get_requisition(w, r) (*(r) = GTK_WIDGET(w)->requisition)
-#define gtk_widget_set_mapped(w, yes) \
-  ((yes) ? GTK_WIDGET_SET_FLAGS(w,GTK_MAPPED) : GTK_WIDGET_UNSET_FLAGS(w,GTK_MAPPED))
-#define gtk_widget_set_realized(w,yes) \
-  ((yes) ? GTK_WIDGET_SET_FLAGS(w,GTK_REALIZED) : GTK_WIDGET_UNSET_FLAGS(w,GTK_REALIZED))
+#define gtk_widget_get_requisition(w,r) (*(r) = GTK_WIDGET(w)->requisition)
+#define gtk_widget_set_mapped(w,yes) { \
+   if (yes) GTK_WIDGET_SET_FLAGS(w,GTK_MAPPED); \
+   else GTK_WIDGET_UNSET_FLAGS(w,GTK_MAPPED); \
+}
+#define gtk_widget_set_realized(w,yes) { \
+   if (yes) GTK_WIDGET_SET_FLAGS(w,GTK_REALIZED); \
+   else GTK_WIDGET_UNSET_FLAGS(w,GTK_REALIZED); \
+}
 #define gtk_range_get_slider_size_fixed(range) (GTK_RANGE(range)->slider_size_fixed)
 #define gtk_range_get_min_slider_size(range) (GTK_RANGE(range)->min_slider_size)
 #define gtk_entry_get_text_window(entry) (GTK_ENTRY(entry)->text_area)
@@ -289,16 +342,24 @@ typedef struct _GtkComboBoxPrivate GtkComboBoxTextPrivate;
 #define gtk_widget_has_grab(wid) ((GTK_WIDGET_FLAGS (wid) & GTK_HAS_GRAB) != 0)
 #define gtk_widget_get_app_paintable(wid) ((GTK_WIDGET_FLAGS (wid) & GTK_APP_PAINTABLE) != 0)
 #define gtk_widget_get_double_buffered(wid) ((GTK_WIDGET_FLAGS (wid) & GTK_DOUBLE_BUFFERED) != 0)
-#define gtk_widget_set_allocation(w, alloc) (GTK_WIDGET(w)->allocation = *(alloc))
-#define gtk_widget_get_allocation(w, alloc) (*(alloc) = GTK_WIDGET(w)->allocation)
-#define gtk_widget_set_can_default(w, yes) \
-  ((yes) ? GTK_WIDGET_SET_FLAGS(w,GTK_CAN_DEFAULT) : GTK_WIDGET_UNSET_FLAGS(w,GTK_CAN_DEFAULT))
-#define gtk_widget_set_can_focus(w, yes) \
-  ((yes) ? GTK_WIDGET_SET_FLAGS(w,GTK_CAN_FOCUS) : GTK_WIDGET_UNSET_FLAGS(w,GTK_CAN_FOCUS))
-#define gtk_widget_set_has_window(w, yes) \
-  ((yes) ? GTK_WIDGET_UNSET_FLAGS(w,GTK_NO_WINDOW) : GTK_WIDGET_SET_FLAGS(w,GTK_NO_WINDOW))
-#define gtk_widget_set_visible(w,yes) \
-  ((yes) ? gtk_widget_show (w) : gtk_widget_hide (w))
+#define gtk_widget_set_allocation(w,alloc) (GTK_WIDGET(w)->allocation = *(alloc))
+#define gtk_widget_get_allocation(w,alloc) (*(alloc) = GTK_WIDGET(w)->allocation)
+#define gtk_widget_set_can_default(w,yes) { \
+   if (yes) GTK_WIDGET_SET_FLAGS(w,GTK_CAN_DEFAULT); \
+   else GTK_WIDGET_UNSET_FLAGS(w,GTK_CAN_DEFAULT); \
+}
+#define gtk_widget_set_can_focus(w,yes) { \
+   if (yes) GTK_WIDGET_SET_FLAGS(w,GTK_CAN_FOCUS); \
+   else GTK_WIDGET_UNSET_FLAGS(w,GTK_CAN_FOCUS); \
+}
+#define gtk_widget_set_has_window(w,yes) { \
+   if (yes) GTK_WIDGET_UNSET_FLAGS(w,GTK_NO_WINDOW); \
+   else GTK_WIDGET_SET_FLAGS(w,GTK_NO_WINDOW); \
+}
+#define gtk_widget_set_visible(w,yes) { \
+   if (yes) gtk_widget_show(w); \
+   else gtk_widget_hide(w); \
+}
 #define gtk_range_get_flippable(range) (GTK_RANGE(range)->flippable)
 #define gdk_window_is_destroyed(w) (GDK_WINDOW_DESTROYED (GDK_WINDOW(w)))
 #endif
@@ -310,6 +371,7 @@ typedef struct _GtkComboBoxPrivate GtkComboBoxTextPrivate;
 #define gtk_menu_item_get_label(i) (gtk_label_get_label (GTK_LABEL (GTK_BIN (i)->child)))
 #define gtk_menu_item_set_label(i,label) gtk_label_set_label(GTK_LABEL(GTK_BIN(i)->child), (label) ? label : "")
 #define gtk_menu_item_get_use_underline(i) (gtk_label_get_use_underline (GTK_LABEL (GTK_BIN (i)->child)))
+#define gtk_status_icon_set_tooltip_text gtk_status_icon_set_tooltip
 #endif
 
 
@@ -338,15 +400,18 @@ typedef struct _GtkComboBoxPrivate GtkComboBoxTextPrivate;
 #	define GDK_KEY_Alt_R GDK_Alt_R
 #	define GDK_KEY_Alt_L GDK_Alt_L
 #	define GDK_KEY_Tab GDK_Tab
-#	define GDK_KEY_Up GDK_Up
 #	define GDK_KEY_space GDK_space
+#	define GDK_KEY_Up GDK_Up
 #	define GDK_KEY_Down GDK_Down
+#	define GDK_KEY_Right GDK_Right
+#	define GDK_KEY_Left GDK_Left
 #	define GDK_KEY_Return GDK_Return
 #	define GDK_KEY_exclam GDK_exclam
 #	define GDK_KEY_BackSpace GDK_BackSpace
 #	define GDK_KEY_Home GDK_Home
 #	define GDK_KEY_End GDK_End
 #	define GDK_KEY_Escape GDK_Escape
+#	define GDK_KEY_Delete GDK_Delete
 #	define GDK_KEY_a GDK_a
 #	define GDK_KEY_A GDK_A
 #	define GDK_KEY_b GDK_b
@@ -399,6 +464,52 @@ typedef struct _GtkComboBoxPrivate GtkComboBoxTextPrivate;
 #	define GDK_KEY_z GDK_z
 #	define GDK_KEY_Z GDK_Z
 #	define GDK_KEY_exclam GDK_exclam
+#	define GDK_KEY_F1 GDK_F1
+#	define GDK_KEY_F2 GDK_F2
+#	define GDK_KEY_F3 GDK_F3
+#	define GDK_KEY_F4 GDK_F4
+#	define GDK_KEY_F5 GDK_F5
+#	define GDK_KEY_F6 GDK_F6
+#	define GDK_KEY_F7 GDK_F7
+#	define GDK_KEY_F8 GDK_F8
+#	define GDK_KEY_F9 GDK_F9
+#	define GDK_KEY_F10 GDK_F10
+#	define GDK_KEY_F11 GDK_F11
+#	define GDK_KEY_F12 GDK_F12
+#	define GDK_KEY_KP_Home GDK_KP_Home
+#	define GDK_KEY_KP_Page_Up GDK_KP_Page_Up
+#	define GDK_KEY_KP_End GDK_KP_End
+#	define GDK_KEY_KP_Page_Down GDK_KP_Page_Down
+#	define GDK_KEY_KP_Space GDK_KP_Space
+#	define GDK_KEY_KP_Enter GDK_KP_Enter
+#	define GDK_KEY_KP_Left GDK_KP_Left
+#	define GDK_KEY_KP_Right GDK_KP_Right
+#	define GDK_KEY_KP_Up GDK_KP_Up
+#	define GDK_KEY_KP_Down GDK_KP_Down
+#	define GDK_KEY_KP_Delete GDK_KP_Delete
+#endif
+
+
+// CAIRO < 1.10
+#if CAIRO_VERSION < CAIRO_VERSION_ENCODE(1, 10, 0)
+#define cairo_region_t         GdkRegion
+#define cairo_rectangle_int_t  GdkRectangle
+#define cairo_region_create    gdk_region_new
+#define cairo_region_copy      gdk_region_copy
+#define cairo_region_destroy   gdk_region_destroy
+#define cairo_region_create_rectangle gdk_region_rectangle
+#define cairo_region_get_extents gdk_region_get_clipbox
+#define cairo_region_is_empty  gdk_region_empty
+#define cairo_region_equal     gdk_region_empty
+#define cairo_region_contains_point gdk_region_point_in
+#define cairo_region_contains_rectangle gdk_region_rect_in
+#define cairo_region_translate gdk_region_rect_in
+#define cairo_region_union_rectangle gdk_region_union_with_rect
+#define cairo_region_intersect gdk_region_intersect
+#define cairo_region_union     gdk_region_union
+#define cairo_region_subtract  gdk_region_subtract
+#define cairo_region_xor       gdk_region_xor
+//#define cairo_region_num_rectangles / cairo_region_get_rectangle  gdk_region_get_rectangles
 #endif
 
 
