@@ -1,27 +1,31 @@
-/*****************************************************************************/
-/*  misc-gtk.c - misc stuff for the gtk+ port of gftp                        */
-/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
-/*                                                                           */
-/*  This program is free software; you can redistribute it and/or modify     */
-/*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 2 of the License, or        */
-/*  (at your option) any later version.                                      */
-/*                                                                           */
-/*  This program is distributed in the hope that it will be useful,          */
-/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
-/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
-/*  GNU General Public License for more details.                             */
-/*                                                                           */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
-/*****************************************************************************/
+/***********************************************************************************/
+/*  misc-gtk.c - misc stuff for the gtk+ port of gftp                              */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                        */
+/*                                                                                 */
+/*  Permission is hereby granted, free of charge, to any person obtaining a copy   */
+/*  of this software and associated documentation files (the "Software"), to deal  */
+/*  in the Software without restriction, including without limitation the rights   */
+/*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      */
+/*  copies of the Software, and to permit persons to whom the Software is          */
+/*  furnished to do so, subject to the following conditions:                       */
+/*                                                                                 */
+/*  The above copyright notice and this permission notice shall be included in all */
+/*  copies or substantial portions of the Software.                                */
+/*                                                                                 */
+/*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     */
+/*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       */
+/*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    */
+/*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         */
+/*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  */
+/*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  */
+/*  SOFTWARE.                                                                      */
+/***********************************************************************************/
 
 #include "gftp-gtk.h"
 
-void
-remove_files_window (gftp_window_data * wdata)
+void remove_files_window (gftp_window_data * wdata)
 {
+  DEBUG_PRINT_FUNC
   wdata->show_selected = 0;
   listbox_clear(wdata);
   free_file_list (wdata->files);
@@ -29,10 +33,10 @@ remove_files_window (gftp_window_data * wdata)
 }
 
 
-void
-ftp_log (gftp_logging_level level, gftp_request * request, 
-         const char *string, ...)
+void ftp_log (gftp_logging_level level, gftp_request * request, 
+              const char *string, ...)
 {
+  //DEBUG_PRINT_FUNC
   uintptr_t max_log_window_size;
   int upd, free_logstr;
   gftp_log * newlog;
@@ -42,7 +46,7 @@ ftp_log (gftp_logging_level level, gftp_request * request,
   size_t len;
   GtkTextBuffer * textbuf;
   GtkTextIter iter, iter2;
-  const char *descr;
+  const char *descr = NULL;
   char *utf8_str;
   size_t destlen;
 
@@ -106,7 +110,9 @@ ftp_log (gftp_logging_level level, gftp_request * request,
 
   gftp_lookup_global_option ("max_log_window_size", &max_log_window_size);
 
-  switch (level)
+  if (gftp_gtk_colored_msgs)
+  {
+    switch (level)
     {
       case gftp_logging_send:
         descr = "send";
@@ -121,7 +127,7 @@ ftp_log (gftp_logging_level level, gftp_request * request,
         descr = "misc";
         break;
     }
-
+  }
   textbuf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (logwdw));
   len = gtk_text_buffer_get_char_count (textbuf);
   gtk_text_buffer_get_iter_at_offset (textbuf, &iter, len);
@@ -147,17 +153,18 @@ ftp_log (gftp_logging_level level, gftp_request * request,
         }
     }
 
+  DEBUG_MSG(logstr)
   if (free_logstr)
     g_free (logstr);
 }
 
 
-void
-update_window_info (void)
+void update_window_info (void)
 {
+  DEBUG_PRINT_FUNC
   char *tempstr, empty[] = "";
   unsigned int port, i, j;
-  GtkWidget *combo_entry;
+  GtkWidget *dir_combo_entry;
 
   if (current_wdata->request != NULL)
     {
@@ -165,13 +172,13 @@ update_window_info (void)
         {
           if ((tempstr = current_wdata->request->hostname) == NULL)
             tempstr = empty;
-          combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(hostedit)));
-          gtk_entry_set_text (GTK_ENTRY (combo_entry), tempstr);
+          dir_combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(hostedit)));
+          gtk_entry_set_text (GTK_ENTRY (dir_combo_entry), tempstr);
 
           if ((tempstr = current_wdata->request->username) == NULL)
             tempstr = empty;
-          combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(useredit)));
-          gtk_entry_set_text (GTK_ENTRY (combo_entry), tempstr);
+          dir_combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(useredit)));
+          gtk_entry_set_text (GTK_ENTRY (dir_combo_entry), tempstr);
     
           if ((tempstr = current_wdata->request->password) == NULL)
             tempstr = empty;
@@ -182,13 +189,13 @@ update_window_info (void)
               port != current_wdata->request->port)
             {
               tempstr = g_strdup_printf ("%d", current_wdata->request->port);
-              combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(portedit)));
-              gtk_entry_set_text (GTK_ENTRY (combo_entry), tempstr);
+              dir_combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(portedit)));
+              gtk_entry_set_text (GTK_ENTRY (dir_combo_entry), tempstr);
               g_free (tempstr);
             }
           else {
-            combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(portedit)));
-            gtk_entry_set_text (GTK_ENTRY (combo_entry), "");
+            dir_combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(portedit)));
+            gtk_entry_set_text (GTK_ENTRY (dir_combo_entry), "");
           }
     
           for (i=0, j=0; gftp_protocols[i].init != NULL; i++)
@@ -219,9 +226,9 @@ update_window_info (void)
 			    && GFTP_IS_CONNECTED (window2.request));
 }
 
-static void
-set_menu_sensitive (gftp_window_data * wdata, char *path, int sensitive)
+static void set_menu_sensitive (gftp_window_data * wdata, char *path, int sensitive)
 {
+  //DEBUG_PRINT_FUNC
   GtkAction *tempwid = NULL;
   if (menus) {
     tempwid = gtk_action_group_get_action(menus, path);
@@ -231,9 +238,9 @@ set_menu_sensitive (gftp_window_data * wdata, char *path, int sensitive)
   }
 }
 
-char *
-report_list_info(gftp_window_data * wdata)
+char * report_list_info(gftp_window_data * wdata)
 {
+  DEBUG_PRINT_FUNC
   GList *templist;
   int  filenum = 0, dirnum = 0, linknum = 0;
   off_t filesize = 0;
@@ -281,46 +288,59 @@ report_list_info(gftp_window_data * wdata)
 }
 
 
-void
-update_window (gftp_window_data * wdata)
+void update_window (gftp_window_data * wdata)
 {
+  DEBUG_PRINT_FUNC
   char *tempstr, *hostname, *fspec, *listinfo;
   int connected;
-  GtkWidget *combo_entry;
-  
+  GtkWidget *dir_combo_entry;
+  char * real_protocol;
+  int i;
+
   connected = GFTP_IS_CONNECTED (wdata->request);
   if (connected)
     {
       fspec = wdata->show_selected ? "Selected" : strcmp (wdata->filespec, "*") == 0 ?  _("All Files") : wdata->filespec;
 
-      if (wdata->request->hostname == NULL ||
-          wdata->request->protonum == GFTP_LOCAL_NUM)
+      if (!wdata->request->hostname || wdata->request->protonum == GFTP_PROTOCOL_LOCALFS) {
         hostname = "";
-      else
+      } else {
         hostname = wdata->request->hostname;
+      }
+      real_protocol = gftp_protocols[wdata->request->protonum].name;
+
+      for (i = 0; gftp_protocols[i].name; i++) {
+         // protocols that are implemented on top of other protocols
+         // may not set the protonum to avoid causing trouble
+         // only the request->url_prefix may provide the true protocol
+         if (wdata->request->url_prefix == gftp_protocols[i].url_prefix) {
+            real_protocol = gftp_protocols[i].name;
+            break;
+         }
+      }
 
       listinfo = report_list_info(wdata);
       tempstr = g_strconcat (hostname, *hostname == '\0' ? "[" : " [",
-                             gftp_protocols[wdata->request->protonum].name,
+                             real_protocol,
                              wdata->request->cached ? _("] (Cached) [") : "] [",
                              fspec, "]",
                              listinfo == NULL ? "" : listinfo,
                              current_wdata == wdata ? "*" : "",
                              NULL);
-      gtk_label_set_text (GTK_LABEL (wdata->hoststxt), tempstr);
+      gtk_label_set_text (GTK_LABEL (wdata->dirinfo_label), tempstr);
       g_free (listinfo);
       g_free (tempstr);
 
       if (wdata->request->directory != NULL) {
-        combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(wdata->combo)));
-        gtk_entry_set_text (GTK_ENTRY (combo_entry), wdata->request->directory);
+        dir_combo_entry = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(wdata->dir_combo)));
+        gtk_entry_set_text (GTK_ENTRY (dir_combo_entry), wdata->request->directory);
       }
     }
-  else if (wdata->hoststxt != NULL)
+  else if (wdata->dirinfo_label != NULL)
     {
       tempstr = g_strconcat (_("Not connected"), 
                              current_wdata == wdata ? "*" : "", NULL);
-      gtk_label_set_text (GTK_LABEL (wdata->hoststxt), tempstr);
+      gtk_label_set_text (GTK_LABEL (wdata->dirinfo_label), tempstr);
       g_free (tempstr);
     }
 
@@ -349,9 +369,9 @@ update_window (gftp_window_data * wdata)
 }  
 
 
-gftp_graphic *
-open_xpm (GtkWidget * widget, char *filename)
+gftp_graphic * open_xpm (GtkWidget * widget, char *filename)
 {
+  //DEBUG_PRINT_FUNC
   gftp_graphic * graphic;
   GtkStyle *style;
   char *exfile;
@@ -384,9 +404,9 @@ open_xpm (GtkWidget * widget, char *filename)
 }
 
 
-void
-gftp_free_pixmap (char *filename)
+void gftp_free_pixmap (char *filename)
 {
+  //DEBUG_PRINT_FUNC
   gftp_graphic * graphic;
 
   if ((graphic = g_hash_table_lookup (graphic_hash_table, filename)) == NULL)
@@ -405,6 +425,7 @@ void
 gftp_get_pixmap (GtkWidget * widget, char *filename, GdkPixmap ** pix,
                  GdkBitmap ** bitmap)
 {
+  //DEBUG_PRINT_FUNC
   gftp_graphic * graphic;
 
   if (filename == NULL || *filename == '\0')
@@ -428,9 +449,9 @@ gftp_get_pixmap (GtkWidget * widget, char *filename, GdkPixmap ** pix,
   *bitmap = graphic->bitmap;
 }
 
-GdkPixbuf *
-gftp_get_pixbuf (char *filename)
+GdkPixbuf * gftp_get_pixbuf (char *filename)
 {
+   //DEBUG_PRINT_FUNC
    GdkPixbuf *pixbuf;
    char *img_path;
 
@@ -461,6 +482,7 @@ check_status (char *name, gftp_window_data *wdata,
               unsigned int check_other_stop, unsigned int only_one,
               unsigned int at_least_one, unsigned int func)
 {
+  DEBUG_PRINT_FUNC
   gftp_window_data * owdata;
 
   owdata = wdata == &window1 ? &window2 : &window1;
@@ -517,6 +539,7 @@ void
 add_history (GtkWidget * widget, GList ** history, unsigned int *histlen, 
              const char *str)
 {
+  DEBUG_PRINT_FUNC
   GList *node, *delnode;
   char *tempstr;
   int i;
@@ -568,51 +591,39 @@ add_history (GtkWidget * widget, GList ** history, unsigned int *histlen,
 }
 
 
-int
-check_reconnect (gftp_window_data *wdata)
-{
+int check_reconnect (gftp_window_data *wdata)
+{ // see gftpui_check_reconnect
+  DEBUG_PRINT_FUNC
   return (wdata->request->cached && wdata->request->datafd < 0 && 
           !wdata->request->always_connected &&
-          !ftp_connect (wdata, wdata->request) ? -1 : 0);
-}
-
-void
-destroy_dialog (gftp_dialog_data * ddata)
-{
-  if (ddata->dialog != NULL)
-    {
-      gtk_widget_destroy (ddata->dialog);
-      ddata->dialog = NULL;
-    }
+          !gftp_gtk_connect (wdata, wdata->request) ? -1 : 0);
 }
 
 
 static void
-dialog_response (GtkWidget * widget, gint response, gftp_dialog_data * ddata)
+dialog_response (GtkDialog * dlg, gint responseID, gpointer data)
 {
-  if (ddata->edit == NULL)
-    {
-      gtk_widget_destroy (ddata->dialog);
-      ddata->dialog = NULL;
-      ddata->checkbox = NULL;
-    }
-
-  switch (response)
-    {
-      case GTK_RESPONSE_YES:
-        if (ddata->yesfunc != NULL)
-          ddata->yesfunc (ddata->yespointer, ddata);
-        break;
-      default:
-        if (ddata->nofunc != NULL)
-          ddata->nofunc (ddata->nopointer, ddata);
-        break;
-    }
-
-  if (ddata->edit != NULL &&
-      ddata->dialog != NULL)
-    gtk_widget_destroy (ddata->dialog);
-
+  //DEBUG_PRINT_FUNC
+  gftp_dialog_data * ddata = (gftp_dialog_data *) data;
+  /* destroy dialog */
+  if (ddata && ddata->dialog) {
+     if (ddata->edit) {
+        ddata->entry_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (ddata->edit)));
+     }
+     if (ddata->checkbox) {
+        ddata->checkbox_is_ticked = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ddata->checkbox));
+     }
+     gtk_widget_destroy (ddata->dialog);
+     ddata->dialog = NULL;
+  }
+  /* run yesfunc or nofunc */
+  if (responseID == GTK_RESPONSE_YES && ddata->yesfunc) {
+     ddata->yesfunc (ddata->yespointer, ddata);
+  } else if (ddata->nofunc) {
+     ddata->nofunc (ddata->nopointer, ddata);
+  }
+  /* free ddata */
+  if (ddata->entry_text) g_free (ddata->entry_text);
   g_free (ddata);
 }
 
@@ -620,17 +631,15 @@ dialog_response (GtkWidget * widget, gint response, gftp_dialog_data * ddata)
 static gint
 dialog_keypress (GtkWidget * widget, GdkEventKey * event, gpointer data)
 {
-  if (event->type != GDK_KEY_PRESS)
-    return (FALSE);
-
+  DEBUG_PRINT_FUNC
   if (event->keyval == GDK_KP_Enter || event->keyval == GDK_Return)
     {
-      dialog_response (widget, GTK_RESPONSE_YES, data);
+      dialog_response (NULL, GTK_RESPONSE_YES, data);
       return (TRUE);
     }
   else if (event->keyval == GDK_Escape)
     {
-      dialog_response (widget, GTK_RESPONSE_NO, data);
+      dialog_response (NULL, GTK_RESPONSE_NO, data);
       return (TRUE);
     }
 
@@ -639,11 +648,14 @@ dialog_keypress (GtkWidget * widget, GdkEventKey * event, gpointer data)
 
 
 void
-MakeEditDialog (char *diagtxt, char *infotxt, char *deftext, int passwd_item,
-                char *checktext, 
-                gftp_dialog_button okbutton, void (*okfunc) (), void *okptr,
-                void (*cancelfunc) (), void *cancelptr)
+TextEntryDialog (GtkWindow * parent_window,       /* nullable */
+                 char * title,   char * infotxt,
+                 char * deftext, int passwd_item,
+                 char * checktext, 
+                 gftp_dialog_button okbutton, void (*okfunc) (), void *okptr,
+                 void (*cancelfunc) (), void *cancelptr)
 {
+  DEBUG_PRINT_FUNC
   GtkWidget * tempwid, * dialog, *vbox;
   gftp_dialog_data * ddata;
   const gchar * yes_text;
@@ -654,6 +666,9 @@ MakeEditDialog (char *diagtxt, char *infotxt, char *deftext, int passwd_item,
   ddata->nofunc = cancelfunc;
   ddata->nopointer = cancelptr;
 
+  dialog = gtk_dialog_new();
+  ddata->dialog = dialog;
+
   switch (okbutton)
     {
       case gftp_dialog_button_ok:
@@ -676,24 +691,30 @@ MakeEditDialog (char *diagtxt, char *infotxt, char *deftext, int passwd_item,
         break;
     }
 
-  dialog = gtk_dialog_new_with_buttons (_(diagtxt),
-                                        NULL,
-                                        GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        "gtk-cancel", GTK_RESPONSE_NO,
-                                        yes_text,         GTK_RESPONSE_YES,
-                                        NULL);
+  if (!parent_window) {
+     // must create a parent window
+     parent_window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
+     gtk_window_set_transient_for (GTK_WINDOW (dialog), parent_window);
+     gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+     gtk_grab_add (dialog);
+  } else {
+     gtk_window_set_transient_for (GTK_WINDOW (dialog), parent_window);
+     gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+     gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
+     gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), TRUE);
+     gtk_window_set_skip_pager_hint (GTK_WINDOW (dialog), TRUE);
+  }
 
-  vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
-
-  gtk_container_set_border_width (GTK_CONTAINER (dialog), 10);
-  gtk_box_set_spacing (GTK_BOX (vbox), 5);
-  gtk_window_set_role (GTK_WINDOW(dialog), "edit");
-  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+  gtk_window_set_title (GTK_WINDOW (dialog), title);
   gtk_widget_set_size_request (dialog, 380, -1);
-  gtk_grab_add (dialog);
-  set_window_icon(GTK_WINDOW(dialog), NULL);
+  set_window_icon (GTK_WINDOW(dialog), NULL);
 
-  ddata->dialog = dialog;
+  gtk_dialog_add_button (GTK_DIALOG (dialog), "gtk-cancel", GTK_RESPONSE_NO);
+  gtk_dialog_add_button (GTK_DIALOG (dialog), yes_text,     GTK_RESPONSE_YES);
+
+  gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
+  vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+  gtk_box_set_spacing (GTK_BOX (vbox), 5);
 
   tempwid = gtk_label_new (infotxt);
   gtk_box_pack_start (GTK_BOX (vbox), tempwid, TRUE, TRUE, 0);
@@ -727,10 +748,12 @@ MakeEditDialog (char *diagtxt, char *infotxt, char *deftext, int passwd_item,
 
 
 void
-MakeYesNoDialog (char *diagtxt, char *infotxt, 
-                 void (*yesfunc) (), gpointer yespointer, 
-                 void (*nofunc) (), gpointer nopointer)
+YesNoDialog (GtkWindow * parent_window,              /* nullable */
+             char * title,       char * infotxt, 
+             void (*yesfunc) (), gpointer yespointer, 
+             void (*nofunc) (),  gpointer nopointer)
 {
+  DEBUG_PRINT_FUNC
   GtkWidget * dialog;
   gftp_dialog_data * ddata;
 
@@ -740,30 +763,28 @@ MakeYesNoDialog (char *diagtxt, char *infotxt,
   ddata->nofunc = nofunc;
   ddata->nopointer = nopointer;
 
-  dialog = gtk_message_dialog_new (NULL,
-                                   GTK_DIALOG_DESTROY_WITH_PARENT,
+  dialog = gtk_message_dialog_new (parent_window,
+                                   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_QUESTION,
                                    GTK_BUTTONS_YES_NO,
                                    "%s", infotxt);
-
-  gtk_window_set_title (GTK_WINDOW (dialog), diagtxt);
-  set_window_icon(GTK_WINDOW(dialog), NULL);
-  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
-  gtk_window_set_role (GTK_WINDOW(dialog), "yndiag");
-  gtk_grab_add (dialog);
-
   ddata->dialog = dialog;
 
+  gtk_window_set_title (GTK_WINDOW (dialog), title);
+  set_window_icon (GTK_WINDOW(dialog), NULL);
+  if (!parent_window) {
+     gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+     gtk_grab_add (dialog);
+  }
   g_signal_connect (G_OBJECT (dialog), "response",
                     G_CALLBACK (dialog_response), ddata);
-
   gtk_widget_show (dialog);
 }
 
 
-void
-display_cached_logs (void)
+void display_cached_logs (void)
 {
+  DEBUG_PRINT_FUNC
   gftp_log * templog;
   GList * templist; 
 
@@ -783,9 +804,9 @@ display_cached_logs (void)
   pthread_mutex_unlock (&log_mutex);
 }
 
-char *
-get_image_path (char *filename)
+char * get_image_path (char *filename)
 {
+   //DEBUG_PRINT_FUNC
    char *path1 = NULL, *path2 = NULL, *found = NULL;
 
    // see lib/misc.c -> gftp_get_share_dir ()
@@ -812,9 +833,9 @@ get_image_path (char *filename)
    return (found);
 }
 
-void
-set_window_icon(GtkWindow *window, char *icon_name)
+void set_window_icon(GtkWindow *window, char *icon_name)
 {
+  //DEBUG_PRINT_FUNC
   GdkPixbuf *pixbuf;
   char *img_path;
   if (icon_name)
@@ -832,7 +853,9 @@ set_window_icon(GtkWindow *window, char *icon_name)
   }
 }
 
-void glist_to_combobox (GList *list, GtkWidget *combo) {
+void glist_to_combobox (GList *list, GtkWidget *combo)
+{
+   //DEBUG_PRINT_FUNC
    GtkTreeIter iter;
    GtkTreeModel *model = GTK_TREE_MODEL (gtk_combo_box_get_model (GTK_COMBO_BOX (combo)));
    GtkListStore *store = GTK_LIST_STORE (model);
@@ -848,11 +871,35 @@ void glist_to_combobox (GList *list, GtkWidget *combo) {
    }
 }
 
+void populate_combo_and_select_protocol (GtkWidget *combo, char * selected_protocol)
+{
+   DEBUG_PRINT_FUNC
+   int i = 0;
+   int combo_item_count = 0;
+   int combo_item_selected = 0;
+   for (i = 0; gftp_protocols[i].name; i++)
+   {
+      if (!gftp_protocols[i].shown) {
+         continue; /* ignore hidden protocols */
+      }
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo),
+                                      gftp_protocols[i].name);
+      if (selected_protocol) {
+         if (strcmp (gftp_protocols[i].name, selected_protocol) == 0) {
+            combo_item_selected = combo_item_count;
+         }
+      }
+      combo_item_count++;
+   }
+   gtk_combo_box_set_active (GTK_COMBO_BOX(combo), combo_item_selected);
+}
+
 
 GtkMenuItem *
 new_menu_item (GtkMenu * menu, char * label, char * icon_name,
                gpointer activate_callback, gpointer callback_data)
 {
+   //DEBUG_PRINT_FUNC
    GtkMenuItem *item = NULL;
 
    /* 0=normal 1=image 2=stock 3=check 4=separator */
