@@ -1,24 +1,27 @@
-/*****************************************************************************/
-/*  menu-items.c - menu callbacks                                            */
-/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
-/*                                                                           */
-/*  This program is free software; you can redistribute it and/or modify     */
-/*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 2 of the License, or        */
-/*  (at your option) any later version.                                      */
-/*                                                                           */
-/*  This program is distributed in the hope that it will be useful,          */
-/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
-/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
-/*  GNU General Public License for more details.                             */
-/*                                                                           */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
-/*****************************************************************************/
+/***********************************************************************************/
+/*  menu-items.c - menu callbacks                                                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                        */
+/*                                                                                 */
+/*  Permission is hereby granted, free of charge, to any person obtaining a copy   */
+/*  of this software and associated documentation files (the "Software"), to deal  */
+/*  in the Software without restriction, including without limitation the rights   */
+/*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      */
+/*  copies of the Software, and to permit persons to whom the Software is          */
+/*  furnished to do so, subject to the following conditions:                       */
+/*                                                                                 */
+/*  The above copyright notice and this permission notice shall be included in all */
+/*  copies or substantial portions of the Software.                                */
+/*                                                                                 */
+/*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     */
+/*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       */
+/*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    */
+/*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         */
+/*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  */
+/*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  */
+/*  SOFTWARE.                                                                      */
+/***********************************************************************************/
 
 #include "gftp-gtk.h"
-#include "options_dialog_localhosts.c"
 
 static gftp_options_dialog_data * gftp_option_data;
 
@@ -628,19 +631,12 @@ clean_old_changes (GtkWidget * widget, gpointer data)
             }
         }
     }
-
-  if (new_proxy_hosts != NULL)
-    {
-      gftp_free_proxy_hosts (new_proxy_hosts);
-      new_proxy_hosts = NULL;
-    }
 }
 
 
 static void
 apply_changes (GtkWidget * widget, gpointer data)
 {
-  gftp_config_list_vars * proxy_hosts;
   gftp_config_vars * cv;
   GList * templist;
   int i;
@@ -662,14 +658,6 @@ apply_changes (GtkWidget * widget, gpointer data)
           gftp_option_types[cv[i].otype].ui_save_function (&cv[i], gftp_option_types[cv[i].otype].user_data);
         }
     }
-
-  gftp_lookup_global_option ("dont_use_proxy", &proxy_hosts);
-
-  if (proxy_hosts->list != NULL)
-    gftp_free_proxy_hosts (proxy_hosts->list);
-
-  proxy_hosts->list = new_proxy_hosts;
-  new_proxy_hosts = NULL;
   gftpui_show_or_hide_command ();
 }
 
@@ -677,15 +665,11 @@ apply_changes (GtkWidget * widget, gpointer data)
 static void
 options_action (GtkWidget * widget, gint response, gpointer user_data)
 {
-  switch (response)
-    {
-      case GTK_RESPONSE_OK:
-        apply_changes (widget, NULL);
-        /* no break */
-      default:
-        clean_old_changes (widget, user_data);
-        gtk_widget_destroy (widget);
-    }
+  switch (response) {
+     case GTK_RESPONSE_OK: apply_changes (widget, NULL); break;
+  }
+  clean_old_changes (widget, user_data);
+  gtk_widget_destroy (widget);
 }
 
 
@@ -782,8 +766,6 @@ options_dialog (gpointer data)
         }
     }
 
-  make_proxy_hosts_tab (gftp_option_data->notebook, gftp_option_data->dialog);
-
   g_signal_connect (G_OBJECT (gftp_option_data->dialog), "response",
                     G_CALLBACK (options_action), NULL);
 
