@@ -1,32 +1,36 @@
-/*****************************************************************************/
-/*  transfer.c - functions to handle transfering files                       */
-/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
-/*                                                                           */
-/*  This program is free software; you can redistribute it and/or modify     */
-/*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 2 of the License, or        */
-/*  (at your option) any later version.                                      */
-/*                                                                           */
-/*  This program is distributed in the hope that it will be useful,          */
-/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
-/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
-/*  GNU General Public License for more details.                             */
-/*                                                                           */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
-/*****************************************************************************/
+/***********************************************************************************/
+/*  transfer.c - functions to handle transfering files                             */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                        */
+/*                                                                                 */
+/*  Permission is hereby granted, free of charge, to any person obtaining a copy   */
+/*  of this software and associated documentation files (the "Software"), to deal  */
+/*  in the Software without restriction, including without limitation the rights   */
+/*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      */
+/*  copies of the Software, and to permit persons to whom the Software is          */
+/*  furnished to do so, subject to the following conditions:                       */
+/*                                                                                 */
+/*  The above copyright notice and this permission notice shall be included in all */
+/*  copies or substantial portions of the Software.                                */
+/*                                                                                 */
+/*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     */
+/*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       */
+/*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    */
+/*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         */
+/*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  */
+/*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  */
+/*  SOFTWARE.                                                                      */
+/***********************************************************************************/
 
 #include "gftp-gtk.h"
 
 static int num_transfers_in_progress = 0;
 
-int
-ftp_list_files (gftp_window_data * wdata)
+int  gftp_gtk_list_files (gftp_window_data * wdata)
 {
+  DEBUG_PRINT_FUNC
   gftpui_callback_data * cdata;
 
-  gtk_label_set_text (GTK_LABEL (wdata->hoststxt), _("Receiving file names..."));
+  gtk_label_set_text (GTK_LABEL (wdata->dirinfo_label), _("Receiving file names..."));
 
   cdata = g_malloc0 (sizeof (*cdata));
   cdata->request = wdata->request;
@@ -57,33 +61,33 @@ ftp_list_files (gftp_window_data * wdata)
 }
 
 
-int
-ftp_connect (gftp_window_data * wdata, gftp_request * request)
+int gftp_gtk_connect (gftp_window_data * wdata, gftp_request * request)
 {
+  DEBUG_PRINT_FUNC
   if (wdata->request == request)
-    gtk_label_set_text (GTK_LABEL (wdata->hoststxt), _("Connecting..."));
+    gtk_label_set_text (GTK_LABEL (wdata->dirinfo_label), _("Connecting..."));
 
   return (gftpui_common_cmd_open (wdata, request, NULL, NULL, NULL));
 }
 
 
-void 
-get_files (gpointer data)
+void  get_files (gpointer data)
 {
+  DEBUG_PRINT_FUNC
   transfer_window_files (&window2, &window1);
 }
 
 
-void
-put_files (gpointer data)
+void put_files (gpointer data)
 {
+  DEBUG_PRINT_FUNC
   transfer_window_files (&window1, &window2);
 }
 
 
-void
-transfer_window_files (gftp_window_data * fromwdata, gftp_window_data * towdata)
+void transfer_window_files (gftp_window_data * fromwdata, gftp_window_data * towdata)
 {
+  DEBUG_PRINT_FUNC
   gftp_file * tempfle, * newfle;
   GList * templist, * igl;
   gftp_transfer * transfer;
@@ -170,6 +174,7 @@ transfer_window_files (gftp_window_data * fromwdata, gftp_window_data * towdata)
 static int
 gftpui_gtk_tdata_connect (gftpui_callback_data * cdata)
 {
+  DEBUG_PRINT_FUNC
   gftp_transfer * tdata;
   int ret;
 
@@ -196,6 +201,7 @@ gftpui_gtk_tdata_connect (gftpui_callback_data * cdata)
 static void
 gftpui_gtk_tdata_disconnect (gftpui_callback_data * cdata)
 {
+  DEBUG_PRINT_FUNC
   gftp_transfer * tdata;
 
   tdata = cdata->user_data;
@@ -213,6 +219,7 @@ gftpui_gtk_tdata_disconnect (gftpui_callback_data * cdata)
 static int
 _gftp_getdir_thread (gftpui_callback_data * cdata)
 {
+  DEBUG_PRINT_FUNC
   return (gftp_get_all_subdirs (cdata->user_data, NULL));
 }
 
@@ -222,9 +229,10 @@ progress_timeout(gpointer data)
   return (1);
 }
 
-int
-gftp_gtk_get_subdirs (gftp_transfer * transfer)
+
+int gftp_gtk_get_subdirs (gftp_transfer * transfer)
 {
+  DEBUG_PRINT_FUNC
   gftpui_callback_data * cdata; 
   long numfiles, numdirs;
   guint timeout_num;
@@ -256,9 +264,9 @@ gftp_gtk_get_subdirs (gftp_transfer * transfer)
 }
 
 
-static void
-remove_file (gftp_viewedit_data * ve_proc)
+static void remove_file (gftp_viewedit_data * ve_proc)
 {
+  DEBUG_PRINT_FUNC
   if (ve_proc->remote_filename == NULL)
     return;
 
@@ -272,9 +280,9 @@ remove_file (gftp_viewedit_data * ve_proc)
 }
 
 
-static void
-free_edit_data (gftp_viewedit_data * ve_proc)
+static void free_edit_data (gftp_viewedit_data * ve_proc)
 {
+  DEBUG_PRINT_FUNC
   int i;
 
   if (ve_proc->torequest)
@@ -293,6 +301,7 @@ free_edit_data (gftp_viewedit_data * ve_proc)
 static void
 dont_upload (gftp_viewedit_data * ve_proc, gftp_dialog_data * ddata)
 {
+  DEBUG_PRINT_FUNC
   remove_file (ve_proc);
   free_edit_data (ve_proc);
 }
@@ -301,6 +310,7 @@ dont_upload (gftp_viewedit_data * ve_proc, gftp_dialog_data * ddata)
 static void
 do_upload (gftp_viewedit_data * ve_proc, gftp_dialog_data * ddata)
 {
+  DEBUG_PRINT_FUNC
   gftp_transfer * tdata;
   gftp_file * tempfle;
   GList * newfile;
@@ -328,6 +338,7 @@ do_upload (gftp_viewedit_data * ve_proc, gftp_dialog_data * ddata)
 static int
 _check_viewedit_process_status (gftp_viewedit_data * ve_proc, int ret)
 {
+  DEBUG_PRINT_FUNC
   int procret;
 
   if (WIFEXITED (ret))
@@ -362,6 +373,7 @@ _check_viewedit_process_status (gftp_viewedit_data * ve_proc, int ret)
 static int
 _prompt_to_upload_edited_file (gftp_viewedit_data * ve_proc)
 {
+  DEBUG_PRINT_FUNC
   struct stat st;
   char *str;
 
@@ -385,8 +397,9 @@ _prompt_to_upload_edited_file (gftp_viewedit_data * ve_proc)
       str = g_strdup_printf (_("File %s has changed.\nWould you like to upload it?"),
                              ve_proc->remote_filename);
 
-      MakeYesNoDialog (_("Edit File"), str, do_upload, ve_proc, dont_upload,
-                       ve_proc);
+      YesNoDialog (NULL, _("Edit File"), str,
+                   do_upload,   ve_proc,
+                   dont_upload, ve_proc);
       g_free (str);
       return (1);
     }
@@ -396,6 +409,7 @@ _prompt_to_upload_edited_file (gftp_viewedit_data * ve_proc)
 static void
 do_check_done_process (pid_t pid, int ret)
 {
+  DEBUG_PRINT_FUNC
   gftp_viewedit_data * ve_proc;
   GList * curdata, *deldata;
   int ok;
@@ -438,6 +452,7 @@ do_check_done_process (pid_t pid, int ret)
 static void
 check_done_process (void)
 {
+  DEBUG_PRINT_FUNC
   pid_t pid;
   int ret;
 
@@ -452,6 +467,7 @@ check_done_process (void)
 static void
 on_next_transfer (gftp_transfer * tdata)
 {
+  DEBUG_PRINT_FUNC
   intptr_t refresh_files;
   gftp_file * tempfle;
 
@@ -491,7 +507,8 @@ on_next_transfer (gftp_transfer * tdata)
 static void
 get_trans_password (gftp_request * request, gftp_dialog_data * ddata)
 {
-  gftp_set_password (request, gtk_entry_get_text (GTK_ENTRY (ddata->edit)));
+  DEBUG_PRINT_FUNC
+  gftp_set_password (request, ddata->entry_text);
   request->stopable = 0;
 }
 
@@ -499,6 +516,7 @@ get_trans_password (gftp_request * request, gftp_dialog_data * ddata)
 static void
 cancel_get_trans_password (gftp_transfer * tdata, gftp_dialog_data * ddata)
 {
+  DEBUG_PRINT_FUNC
   if (tdata->fromreq->stopable == 0)
     return;
 
@@ -509,6 +527,7 @@ cancel_get_trans_password (gftp_transfer * tdata, gftp_dialog_data * ddata)
 static void
 show_transfer (gftp_transfer * tdata)
 {
+  DEBUG_PRINT_FUNC
   GdkPixmap * closedir_pixmap, * opendir_pixmap;
   GdkBitmap * closedir_bitmap, * opendir_bitmap;
   gftpui_common_curtrans_data * transdata;
@@ -562,21 +581,21 @@ show_transfer (gftp_transfer * tdata)
   if (!tdata->toreq->stopable && gftp_need_password (tdata->toreq))
     {
       tdata->toreq->stopable = 1;
-      MakeEditDialog (_("Enter Password"),
-                      _("Please enter your password for this site"), NULL, 0,
-                      NULL, gftp_dialog_button_connect, 
-                      get_trans_password, tdata->toreq,
-                      cancel_get_trans_password, tdata);
+      TextEntryDialog (NULL, _("Enter Password"),
+                       _("Please enter your password for this site"), NULL, 0,
+                       NULL, gftp_dialog_button_connect, 
+                       get_trans_password, tdata->toreq,
+                       cancel_get_trans_password, tdata);
     }
 
   if (!tdata->fromreq->stopable && gftp_need_password (tdata->fromreq))
     {
       tdata->fromreq->stopable = 1;
-      MakeEditDialog (_("Enter Password"),
-                      _("Please enter your password for this site"), NULL, 0,
-                      NULL, gftp_dialog_button_connect, 
-                      get_trans_password, tdata->fromreq,
-                      cancel_get_trans_password, tdata);
+      TextEntryDialog (NULL, _("Enter Password"),
+                       _("Please enter your password for this site"), NULL, 0,
+                       NULL, gftp_dialog_button_connect, 
+                       get_trans_password, tdata->fromreq,
+                       cancel_get_trans_password, tdata);
     }
 }
 
@@ -584,6 +603,7 @@ show_transfer (gftp_transfer * tdata)
 static void
 transfer_done (GList * node)
 {
+  DEBUG_PRINT_FUNC
   gftpui_common_curtrans_data * transdata;
   gftp_transfer * tdata;
   gftp_file * tempfle;
@@ -655,8 +675,8 @@ transfer_done (GList * node)
 static void *
 _gftpui_transfer_files (void *data)
 {
+  DEBUG_PRINT_FUNC
   int ret;
-
   pthread_detach (pthread_self ());
   ret = gftpui_common_transfer_files (data);
   return (GINT_TO_POINTER(ret));
@@ -666,6 +686,7 @@ _gftpui_transfer_files (void *data)
 static void
 create_transfer (gftp_transfer * tdata)
 {
+  DEBUG_PRINT_FUNC
   if (tdata->fromreq->stopable)
     return;
 
@@ -703,6 +724,7 @@ static void
 _setup_dlstr (gftp_transfer * tdata, gftp_file * fle, char *dlstr,
               size_t dlstr_len)
 {
+  DEBUG_PRINT_FUNC
   int hours, mins, secs, stalled, usesentdescr;
   unsigned long remaining_secs, lkbs;
   char gotstr[50], ofstr[50];
@@ -710,7 +732,7 @@ _setup_dlstr (gftp_transfer * tdata, gftp_file * fle, char *dlstr,
 
   stalled = 1;
   gettimeofday (&tv, NULL);
-  usesentdescr = (tdata->fromreq->protonum == GFTP_LOCAL_NUM);
+  usesentdescr = (tdata->fromreq->protonum == GFTP_PROTOCOL_LOCALFS);
 
   insert_commas (fle->size, ofstr, sizeof (ofstr));
   insert_commas (tdata->curtrans + tdata->curresumed, gotstr, sizeof (gotstr));
@@ -767,6 +789,7 @@ _setup_dlstr (gftp_transfer * tdata, gftp_file * fle, char *dlstr,
 static void
 update_file_status (gftp_transfer * tdata)
 {
+  DEBUG_PRINT_FUNC
   char totstr[150], winstr[150], dlstr[150];
   unsigned long remaining_secs, lkbs;
   int hours, mins, secs, pcent;
@@ -833,6 +856,7 @@ update_file_status (gftp_transfer * tdata)
 static void
 update_window_transfer_bytes (gftp_window_data * wdata)
 {
+  DEBUG_PRINT_FUNC
   char *tempstr, *temp1str;
 
   if (wdata->request->gotbytes == -1)
@@ -845,7 +869,7 @@ update_window_transfer_bytes (gftp_window_data * wdata)
       tempstr = insert_commas (wdata->request->gotbytes, NULL, 0);
       temp1str = g_strdup_printf (_("Retrieving file names...%s bytes"), 
                                   tempstr);
-      gtk_label_set_text (GTK_LABEL (wdata->hoststxt), temp1str);
+      gtk_label_set_text (GTK_LABEL (wdata->dirinfo_label), temp1str);
       g_free (tempstr);
       g_free (temp1str);
     }
@@ -855,6 +879,7 @@ update_window_transfer_bytes (gftp_window_data * wdata)
 gint
 update_downloads (gpointer data)
 {
+  //DEBUG_PRINT_FUNC /* endless loop 1 second */
   intptr_t do_one_transfer_at_a_time, start_transfers;
   GList * templist, * next;
   gftp_transfer * tdata;
@@ -916,6 +941,7 @@ update_downloads (gpointer data)
 void
 start_transfer (gpointer data)
 {
+  DEBUG_PRINT_FUNC
   gftpui_common_curtrans_data * transdata;
   GtkCTreeNode * node;
 
@@ -938,6 +964,7 @@ start_transfer (gpointer data)
 void
 stop_transfer (gpointer data)
 {
+  DEBUG_PRINT_FUNC
   gftpui_common_curtrans_data * transdata;
   GtkCTreeNode * node;
 
@@ -957,6 +984,7 @@ stop_transfer (gpointer data)
 void
 skip_transfer (gpointer data)
 {
+  DEBUG_PRINT_FUNC
   gftpui_common_curtrans_data * transdata;
   GtkCTreeNode * node;
 
@@ -978,6 +1006,7 @@ skip_transfer (gpointer data)
 void
 remove_file_transfer (gpointer data)
 {
+  DEBUG_PRINT_FUNC
   gftpui_common_curtrans_data * transdata;
   GtkCTreeNode * node;
   gftp_file * curfle;
@@ -1006,6 +1035,7 @@ remove_file_transfer (gpointer data)
 void
 move_transfer_up (gpointer data)
 {
+  DEBUG_PRINT_FUNC
   GList * firstentry, * secentry, * lastentry;
   gftpui_common_curtrans_data * transdata;
   GtkCTreeNode * node;
@@ -1066,6 +1096,7 @@ move_transfer_up (gpointer data)
 void
 move_transfer_down (gpointer data)
 {
+  DEBUG_PRINT_FUNC
   GList * firstentry, * secentry, * lastentry;
   gftpui_common_curtrans_data * transdata;
   GtkCTreeNode * node;
