@@ -1,21 +1,25 @@
-/*****************************************************************************/
-/*  gftpui.c - UI related functions for gFTP                                 */
-/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
-/*                                                                           */
-/*  This program is free software; you can redistribute it and/or modify     */
-/*  it under the terms of the GNU General Public License as published by     */
-/*  the Free Software Foundation; either version 2 of the License, or        */
-/*  (at your option) any later version.                                      */
-/*                                                                           */
-/*  This program is distributed in the hope that it will be useful,          */
-/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
-/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
-/*  GNU General Public License for more details.                             */
-/*                                                                           */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program; if not, write to the Free Software              */
-/*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA      */
-/*****************************************************************************/
+/***********************************************************************************/
+/*  gftpui.c - UI related functions for gFTP                                       */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                        */
+/*                                                                                 */
+/*  Permission is hereby granted, free of charge, to any person obtaining a copy   */
+/*  of this software and associated documentation files (the "Software"), to deal  */
+/*  in the Software without restriction, including without limitation the rights   */
+/*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      */
+/*  copies of the Software, and to permit persons to whom the Software is          */
+/*  furnished to do so, subject to the following conditions:                       */
+/*                                                                                 */
+/*  The above copyright notice and this permission notice shall be included in all */
+/*  copies or substantial portions of the Software.                                */
+/*                                                                                 */
+/*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     */
+/*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       */
+/*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    */
+/*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         */
+/*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  */
+/*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  */
+/*  SOFTWARE.                                                                      */
+/***********************************************************************************/
 
 #include "gftpui.h"
 
@@ -27,6 +31,7 @@ static gftp_logging_func gftpui_common_logfunc;
 static void *
 _gftpui_common_thread_callback (void * data)
 { 
+  DEBUG_PRINT_FUNC
   intptr_t network_timeout, sleep_time;
   gftpui_callback_data * cdata;
   struct timespec ts;
@@ -79,6 +84,7 @@ _gftpui_common_thread_callback (void * data)
 int
 gftpui_common_run_callback_function (gftpui_callback_data * cdata)
 {
+  DEBUG_PRINT_FUNC
   int ret;
 
   if (!cdata->dont_check_connection && gftpui_check_reconnect (cdata) < 0)
@@ -102,6 +108,7 @@ gftpui_common_run_callback_function (gftpui_callback_data * cdata)
 static void
 gftpui_common_signal_handler (int signo)
 {
+  DEBUG_PRINT_FUNC
   signal (signo, gftpui_common_signal_handler);
 
   if (!gftpui_common_num_child_threads && signo == SIGINT)
@@ -112,6 +119,7 @@ gftpui_common_signal_handler (int signo)
 static void
 gftpui_common_sig_child (int signo)
 {
+  DEBUG_PRINT_FUNC
   int ret;
 
   if (gftpui_common_child_process_done == -1)
@@ -130,6 +138,7 @@ gftpui_common_sig_child (int signo)
 void
 gftpui_common_init (int *argc, char ***argv, gftp_logging_func logfunc)
 {
+  DEBUG_PRINT_FUNC
   char *share_dir;
 
   gftp_locale_init ();
@@ -152,13 +161,8 @@ gftpui_common_init (int *argc, char ***argv, gftp_logging_func logfunc)
 void
 gftpui_common_about (gftp_logging_func logging_function, gpointer logdata)
 {
-  char *str;
-
+  DEBUG_PRINT_FUNC
   logging_function (gftp_logging_misc, logdata, "%s - http://www.gftp.org/\n", gftp_version);
-
-  str = _("Translated by");
-  if (strcmp (str, "Translated by") != 0)
-    logging_function (gftp_logging_misc, logdata, "%s\n", str);
 }
 
 
@@ -167,6 +171,7 @@ gftpui_common_cmd_about (void *uidata, gftp_request * request,
                          void *other_uidata, gftp_request * other_request,
                          const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftpui_common_about (gftpui_common_logfunc, NULL);
   return (1);
 }
@@ -177,6 +182,7 @@ gftpui_common_cmd_ascii (void *uidata, gftp_request * request,
                          void *other_uidata, gftp_request * other_request,
                          const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftp_set_global_option ("ascii_transfers", GINT_TO_POINTER(1));
   return (1);
 }
@@ -187,6 +193,7 @@ gftpui_common_cmd_binary (void *uidata, gftp_request * request,
                          void *other_uidata, gftp_request * other_request,
                          const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftp_set_global_option ("ascii_transfers", GINT_TO_POINTER(0));
   return (1);
 }
@@ -197,6 +204,7 @@ gftpui_common_cmd_chmod (void *uidata, gftp_request * request,
                          void *other_uidata, gftp_request * other_request,
                          const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftpui_callback_data * cdata;
   char *pos;
 
@@ -239,6 +247,7 @@ gftpui_common_cmd_rename (void *uidata, gftp_request * request,
                           void *other_uidata, gftp_request * other_request,
                           const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftpui_callback_data * cdata;
   char *pos;
   
@@ -280,6 +289,7 @@ gftpui_common_cmd_delete (void *uidata, gftp_request * request,
                           void *other_uidata, gftp_request * other_request,
                           const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftpui_callback_data * cdata;
 
   if (!GFTP_IS_CONNECTED (request))
@@ -315,6 +325,7 @@ gftpui_common_cmd_rmdir (void *uidata, gftp_request * request,
                          void *other_uidata, gftp_request * other_request,
                          const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftpui_callback_data * cdata;
 
   if (!GFTP_IS_CONNECTED (request))
@@ -350,6 +361,7 @@ gftpui_common_cmd_site (void *uidata, gftp_request * request,
                         void *other_uidata, gftp_request * other_request,
                         const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftpui_callback_data * cdata;
 
   if (!GFTP_IS_CONNECTED (request))
@@ -386,6 +398,7 @@ gftpui_common_cmd_mkdir (void *uidata, gftp_request * request,
                          void *other_uidata, gftp_request * other_request,
                          const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftpui_callback_data * cdata;
 
   if (!GFTP_IS_CONNECTED (request))
@@ -421,6 +434,7 @@ gftpui_common_cmd_chdir (void *uidata, gftp_request * request,
                          void *other_uidata, gftp_request * other_request,
                          const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftpui_callback_data * cdata;
   char *tempstr, *newdir = NULL;
 
@@ -436,7 +450,7 @@ gftpui_common_cmd_chdir (void *uidata, gftp_request * request,
                                  _("usage: chdir <directory>\n"));
       return (1);
     }
-  else if (request->protonum == GFTP_LOCAL_NUM)
+  else if (request->protonum == GFTP_PROTOCOL_LOCALFS)
     {
       if (*command != '/' && request->directory != NULL)
         {
@@ -479,6 +493,7 @@ gftpui_common_cmd_close (void *uidata, gftp_request * request,
                          void *other_uidata, gftp_request * other_request,
                          const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftp_disconnect (request);
   return (1);
 }
@@ -489,6 +504,7 @@ gftpui_common_cmd_pwd (void *uidata, gftp_request * request,
                        void *other_uidata, gftp_request * other_request,
                        const char *command)
 {
+  DEBUG_PRINT_FUNC
   if (!GFTP_IS_CONNECTED (request))
     {
       request->logging_function (gftp_logging_error, request,
@@ -508,6 +524,7 @@ gftpui_common_cmd_quit (void *uidata, gftp_request * request,
                         void *other_uidata, gftp_request * other_request,
                         const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftp_shutdown();
 
   return (0);
@@ -519,6 +536,7 @@ gftpui_common_cmd_clear (void *uidata, gftp_request * request,
                          void *other_uidata, gftp_request * other_request,
                          const char *command)
 {
+  DEBUG_PRINT_FUNC
   if (strcasecmp (command, "cache") == 0)
     gftp_clear_cache_files ();
   else
@@ -534,6 +552,7 @@ gftpui_common_cmd_clear (void *uidata, gftp_request * request,
 static int
 gftpui_common_clear_show_subhelp (const char *topic)
 {
+  DEBUG_PRINT_FUNC
   if (strcmp (topic, "cache") == 0)
     {
       gftpui_common_logfunc (gftp_logging_misc, NULL,
@@ -548,6 +567,7 @@ gftpui_common_clear_show_subhelp (const char *topic)
 static int
 gftpui_common_set_show_subhelp (const char *topic)
 { 
+  DEBUG_PRINT_FUNC
   gftp_config_vars * cv;
     
   if ((cv = g_hash_table_lookup (gftp_global_options_htable, topic)) != NULL)
@@ -565,6 +585,7 @@ gftpui_common_cmd_ls (void *uidata, gftp_request * request,
                       void *other_uidata, gftp_request * other_request,
                       const char *command)
 {
+  DEBUG_PRINT_FUNC
   char *startcolor, *endcolor, *tempstr;
   gftpui_callback_data * cdata;
   GList * templist;
@@ -615,6 +636,7 @@ gftpui_common_cmd_open (void *uidata, gftp_request * request,
                         void *other_uidata, gftp_request * other_request,
                         const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftpui_callback_data * cdata;
   intptr_t retries;
 
@@ -667,6 +689,7 @@ gftpui_common_cmd_set (void *uidata, gftp_request * request,
                        void *other_uidata, gftp_request * other_request,
                        const char *command)
 {
+  DEBUG_PRINT_FUNC
   char *pos, *backpos, buf[256];
   gftp_config_vars * cv, newcv;
   GList * templist;
@@ -758,6 +781,7 @@ gftpui_common_cmd_help (void *uidata, gftp_request * request,
                         void *other_uidata, gftp_request * other_request,
                         const char *command)
 {
+  DEBUG_PRINT_FUNC
   int i, j, ele, numrows, numcols = 6, handled, number_commands, cmdlen,
       found;
   char commands[128], cmdstr[30];
@@ -836,6 +860,7 @@ _gftpui_common_cmd_transfer_files (void *fromuidata, gftp_request * fromrequest,
                                    void *touidata, gftp_request * torequest,
                                    const char *cmd, const char *filespec)
 {
+  DEBUG_PRINT_FUNC
   gftp_transfer * tdata;
   gftp_file * fle;
 
@@ -917,6 +942,7 @@ gftpui_common_cmd_mget_file (void *uidata, gftp_request * request,
                              void *other_uidata, gftp_request * other_request,
                              const char *command)
 {
+  DEBUG_PRINT_FUNC
   _gftpui_common_cmd_transfer_files (uidata, request, other_uidata,
                                      other_request, "mget", command);
   return (1);
@@ -928,6 +954,7 @@ gftpui_common_cmd_mput_file (void *uidata, gftp_request * request,
                              void *other_uidata, gftp_request * other_request,
                              const char *command)
 {
+  DEBUG_PRINT_FUNC
   _gftpui_common_cmd_transfer_files (other_uidata, other_request, uidata,
                                      request, "mput", command);
   return (1);
@@ -935,76 +962,76 @@ gftpui_common_cmd_mput_file (void *uidata, gftp_request * request,
 
 
 gftpui_common_methods gftpui_common_commands[] = {
-        {N_("about"),   2, gftpui_common_cmd_about, gftpui_common_request_none,
+        { "about",   2, gftpui_common_cmd_about, gftpui_common_request_none,
          N_("Shows gFTP information"), NULL},
-        {N_("ascii"),   2, gftpui_common_cmd_ascii, gftpui_common_request_remote,
+        { "ascii",   2, gftpui_common_cmd_ascii, gftpui_common_request_remote,
          N_("Sets the current file transfer mode to Ascii (only for FTP)"), NULL},
-        {N_("binary"),  1, gftpui_common_cmd_binary, gftpui_common_request_remote,
+        { "binary",  1, gftpui_common_cmd_binary, gftpui_common_request_remote,
          N_("Sets the current file transfer mode to Binary (only for FTP)"), NULL},
-        {N_("cd"),      2, gftpui_common_cmd_chdir, gftpui_common_request_remote,
+        { "cd",      2, gftpui_common_cmd_chdir, gftpui_common_request_remote,
          N_("Changes the remote working directory"), NULL},
-        {N_("chdir"),   3, gftpui_common_cmd_chdir, gftpui_common_request_remote,
+        { "chdir",   3, gftpui_common_cmd_chdir, gftpui_common_request_remote,
          N_("Changes the remote working directory"), NULL},
-        {N_("chmod"),   3, gftpui_common_cmd_chmod, gftpui_common_request_remote,
+        { "chmod",   3, gftpui_common_cmd_chmod, gftpui_common_request_remote,
          N_("Changes the permissions of a remote file"), NULL},
-        {N_("clear"),   3, gftpui_common_cmd_clear, gftpui_common_request_none,
+        { "clear",   3, gftpui_common_cmd_clear, gftpui_common_request_none,
          N_("Available options: cache"), gftpui_common_clear_show_subhelp},
-        {N_("close"),   3, gftpui_common_cmd_close, gftpui_common_request_remote,
+        { "close",   3, gftpui_common_cmd_close, gftpui_common_request_remote,
          N_("Disconnects from the remote site"), NULL},
-        {N_("delete"),  1, gftpui_common_cmd_delete, gftpui_common_request_remote,
+        { "delete",  1, gftpui_common_cmd_delete, gftpui_common_request_remote,
          N_("Removes a remote file"), NULL},
-        {N_("dir"),     3, gftpui_common_cmd_ls, gftpui_common_request_remote,
+        { "dir",     3, gftpui_common_cmd_ls, gftpui_common_request_remote,
          N_("Shows the directory listing for the current remote directory"), NULL},
-        {N_("get"),     1, gftpui_common_cmd_mget_file, gftpui_common_request_remote,
+        { "get",     1, gftpui_common_cmd_mget_file, gftpui_common_request_remote,
          N_("Downloads remote file(s)"), NULL},
-        {N_("help"),    1, gftpui_common_cmd_help, gftpui_common_request_none,
+        { "help",    1, gftpui_common_cmd_help, gftpui_common_request_none,
          N_("Shows this help screen"), NULL},
-        {N_("lcd"),     3, gftpui_common_cmd_chdir, gftpui_common_request_local,
+        { "lcd",     3, gftpui_common_cmd_chdir, gftpui_common_request_local,
          N_("Changes the local working directory"), NULL},
-        {N_("lchdir"),  4, gftpui_common_cmd_chdir, gftpui_common_request_local,
+        { "lchdir",  4, gftpui_common_cmd_chdir, gftpui_common_request_local,
          N_("Changes the local working directory"), NULL},
-        {N_("lchmod"),  4, gftpui_common_cmd_chmod,     gftpui_common_request_local,
+        { "lchmod",  4, gftpui_common_cmd_chmod,     gftpui_common_request_local,
          N_("Changes the permissions of a local file"), NULL},
-        {N_("ldelete"), 2, gftpui_common_cmd_delete,    gftpui_common_request_local,
+        { "ldelete", 2, gftpui_common_cmd_delete,    gftpui_common_request_local,
          N_("Removes a local file"), NULL},
-        {N_("ldir"),    4, gftpui_common_cmd_ls, gftpui_common_request_local,
+        { "ldir",    4, gftpui_common_cmd_ls, gftpui_common_request_local,
          N_("Shows the directory listing for the current local directory"), NULL},
-        {N_("lls"),     2, gftpui_common_cmd_ls, gftpui_common_request_local,
+        { "lls",     2, gftpui_common_cmd_ls, gftpui_common_request_local,
          N_("Shows the directory listing for the current local directory"), NULL},
-        {N_("lmkdir"),  2, gftpui_common_cmd_mkdir, gftpui_common_request_local,
+        { "lmkdir",  2, gftpui_common_cmd_mkdir, gftpui_common_request_local,
          N_("Creates a local directory"), NULL},
-        {N_("lpwd"),    2, gftpui_common_cmd_pwd, gftpui_common_request_local,
+        { "lpwd",    2, gftpui_common_cmd_pwd, gftpui_common_request_local,
          N_("Show current local directory"), NULL},
-        {N_("lrename"), 3, gftpui_common_cmd_rename, gftpui_common_request_local,
+        { "lrename", 3, gftpui_common_cmd_rename, gftpui_common_request_local,
          N_("Rename a local file"), NULL},
-        {N_("lrmdir"),  3, gftpui_common_cmd_rmdir, gftpui_common_request_local,
+        { "lrmdir",  3, gftpui_common_cmd_rmdir, gftpui_common_request_local,
          N_("Remove a local directory"), NULL},
-        {N_("ls"),      2, gftpui_common_cmd_ls, gftpui_common_request_remote,
+        { "ls",      2, gftpui_common_cmd_ls, gftpui_common_request_remote,
          N_("Shows the directory listing for the current remote directory"), NULL},
-        {N_("mget"),    2, gftpui_common_cmd_mget_file, gftpui_common_request_remote,
+        { "mget",    2, gftpui_common_cmd_mget_file, gftpui_common_request_remote,
          N_("Downloads remote file(s)"), NULL},
-        {N_("mkdir"),   2, gftpui_common_cmd_mkdir, gftpui_common_request_remote,
+        { "mkdir",   2, gftpui_common_cmd_mkdir, gftpui_common_request_remote,
          N_("Creates a remote directory"), NULL},
-        {N_("mput"),    2, gftpui_common_cmd_mput_file, gftpui_common_request_remote,
+        { "mput",    2, gftpui_common_cmd_mput_file, gftpui_common_request_remote,
          N_("Uploads local file(s)"), NULL},
-        {N_("open"),    1, gftpui_common_cmd_open, gftpui_common_request_remote,
+        { "open",    1, gftpui_common_cmd_open, gftpui_common_request_remote,
          N_("Opens a connection to a remote site"), NULL},
-        {N_("put"),     2, gftpui_common_cmd_mput_file, gftpui_common_request_remote,
+        { "put",     2, gftpui_common_cmd_mput_file, gftpui_common_request_remote,
          N_("Uploads local file(s)"), NULL},
-        {N_("pwd"),     2, gftpui_common_cmd_pwd, gftpui_common_request_remote,
+        { "pwd",     2, gftpui_common_cmd_pwd, gftpui_common_request_remote,
          N_("Show current remote directory"), NULL},
-        {N_("quit"),    1, gftpui_common_cmd_quit, gftpui_common_request_none,
+        { "quit",    1, gftpui_common_cmd_quit, gftpui_common_request_none,
          N_("Exit from gFTP"), NULL},
-        {N_("rename"),  2, gftpui_common_cmd_rename, gftpui_common_request_remote,
+        { "rename",  2, gftpui_common_cmd_rename, gftpui_common_request_remote,
          N_("Rename a remote file"), NULL},
-        {N_("rmdir"),   2, gftpui_common_cmd_rmdir, gftpui_common_request_remote,
+        { "rmdir",   2, gftpui_common_cmd_rmdir, gftpui_common_request_remote,
          N_("Remove a remote directory"), NULL},
-        {N_("set"),     1, gftpui_common_cmd_set, gftpui_common_request_none,
+        { "set",     1, gftpui_common_cmd_set, gftpui_common_request_none,
          N_("Show configuration file variables. You can also set variables by set var=val"),
          gftpui_common_set_show_subhelp},
-        {N_("site"),    2, gftpui_common_cmd_site, gftpui_common_request_remote,
+        { "site",    2, gftpui_common_cmd_site, gftpui_common_request_remote,
          N_("Run a site specific command"), NULL},
-        {NULL,          0, NULL,                gftpui_common_request_none,
+        { NULL,          0, NULL,                gftpui_common_request_none,
 	 NULL, NULL}};
 
 
@@ -1013,6 +1040,7 @@ gftpui_common_process_command (void *locui, gftp_request * locreq,
                                void *remui, gftp_request * remreq,
                                const char *command)
 {
+  DEBUG_PRINT_FUNC
   gftp_request * request, * other_request;
   void *uidata, *other_uidata;
   char *pos, *newstr;
@@ -1106,6 +1134,7 @@ gftpui_common_add_file_transfer (gftp_request * fromreq, gftp_request * toreq,
                                  void *fromuidata, void *touidata,
                                  GList * files)
 {
+  DEBUG_PRINT_FUNC
   intptr_t append_transfers, one_transfer, overwrite_default;
   GList * templist, *curfle;
   gftp_transfer * tdata;
@@ -1223,6 +1252,7 @@ static ssize_t
 _do_transfer_block (gftp_transfer * tdata, gftp_file * curfle, char *buf,
                     size_t trans_blksize)
 {
+  DEBUG_PRINT_FUNC
   ssize_t num_read, num_wrote, ret;
   char *bufpos;
 
@@ -1249,6 +1279,7 @@ _do_transfer_block (gftp_transfer * tdata, gftp_file * curfle, char *buf,
 int
 _gftpui_common_do_transfer_file (gftp_transfer * tdata, gftp_file * curfle)
 {
+  DEBUG_PRINT_FUNC
   struct timeval updatetime;
   intptr_t trans_blksize;
   ssize_t num_trans;
@@ -1308,6 +1339,7 @@ _gftpui_common_do_transfer_file (gftp_transfer * tdata, gftp_file * curfle)
 void
 gftpui_common_skip_file_transfer (gftp_transfer * tdata, gftp_file * curfle)
 {
+  DEBUG_PRINT_FUNC
   Wg_mutex_lock (&tdata->structmutex);
 
   if (tdata->started && !(curfle->transfer_action & GFTP_TRANS_ACTION_SKIP))
@@ -1334,6 +1366,7 @@ gftpui_common_skip_file_transfer (gftp_transfer * tdata, gftp_file * curfle)
 void
 gftpui_common_cancel_file_transfer (gftp_transfer * tdata)
 {
+  DEBUG_PRINT_FUNC
   Wg_mutex_lock (&tdata->structmutex);
 
   if (tdata->started)
@@ -1358,6 +1391,7 @@ gftpui_common_cancel_file_transfer (gftp_transfer * tdata)
 static void
 _gftpui_common_next_file_in_trans (gftp_transfer * tdata)
 {
+  DEBUG_PRINT_FUNC
   gftp_file * curfle;
 
   Wg_mutex_lock (&tdata->structmutex);
@@ -1376,6 +1410,7 @@ _gftpui_common_next_file_in_trans (gftp_transfer * tdata)
 static int
 _gftpui_common_preserve_perm_time (gftp_transfer * tdata, gftp_file * curfle)
 {
+  DEBUG_PRINT_FUNC
   intptr_t preserve_permissions, preserve_time;
   int ret, tmpret;
 
@@ -1413,6 +1448,7 @@ _gftpui_common_preserve_perm_time (gftp_transfer * tdata, gftp_file * curfle)
 static int
 _gftpui_common_trans_file_or_dir (gftp_transfer * tdata)
 {
+  DEBUG_PRINT_FUNC
   gftp_file * curfle;
   int ret;
 
@@ -1448,9 +1484,10 @@ _gftpui_common_trans_file_or_dir (gftp_transfer * tdata)
       if (curfle->size == 0)
         {
           curfle->size = gftp_get_file_size (tdata->fromreq, curfle->file);
-          if (curfle->size < 0)
+          if (curfle->size < 0) {
+            DEBUG_MSG("filesize < 0!!!!!!!!!!\n\n")
             return ((int) curfle->size);
-
+          }
           tdata->total_bytes += curfle->size;
         }
 
@@ -1458,8 +1495,10 @@ _gftpui_common_trans_file_or_dir (gftp_transfer * tdata)
         {
           curfle->transfer_action = GFTP_TRANS_ACTION_RESUME;
           curfle->startsize = gftp_get_file_size (tdata->toreq, curfle->destfile);
-          if (curfle->startsize < 0)
+          if (curfle->startsize < 0) {
+            DEBUG_MSG("(retry) filesize < 0!!!!!!!!!!\n\n")
             return ((int) curfle->startsize);
+          }
         }
 
       tdata->tot_file_trans = gftp_transfer_file (tdata->fromreq, curfle->file,
@@ -1501,6 +1540,7 @@ _gftpui_common_trans_file_or_dir (gftp_transfer * tdata)
 int
 gftpui_common_transfer_files (gftp_transfer * tdata)
 {
+  DEBUG_PRINT_FUNC
   int ret, skipped_files;
 
   tdata->curfle = tdata->files;
@@ -1546,7 +1586,7 @@ gftpui_common_transfer_files (gftp_transfer * tdata)
 
   if (skipped_files)
     tdata->fromreq->logging_function (gftp_logging_error, tdata->fromreq,
-                                      _("There were %d files or directories that could not be transferred. Check the log for which items were not properly transferred."),
+                                      _("There were %d files or directories that could not be transferred. Check the log for which items were not properly transferred.\n"),
                                       skipped_files);
 
   tdata->done = 1;
@@ -1559,6 +1599,7 @@ gftpui_common_transfer_files (gftp_transfer * tdata)
 void
 gftpui_protocol_update_timeout (gftp_request * request)
 {
+  DEBUG_PRINT_FUNC
   intptr_t network_timeout;
 
   gftp_lookup_request_option (request, "network_timeout", &network_timeout);
